Package: primirTSS
Title: Prediction of pri-miRNA Transcription Start Site
Version: 1.29.0
Author: Pumin Li [aut, cre], Qi Xu [aut], Jie Li [aut], Jin Wang [aut]
Maintainer: Pumin Li <ipumin@163.com>
Description: A fast, convenient tool to identify the TSSs of miRNAs by 
    integrating the data of H3K4me3 and Pol II as well as combining the 
    conservation level and sequence feature, provided within both command-line 
    and graphical interfaces, which achieves a better performance than the 
    previous non-cell-specific methods on miRNA TSSs.
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.1.0
Depends: R (>= 3.5.0)
Imports: GenomicRanges (>= 1.32.2), S4Vectors (>= 0.18.2), rtracklayer
        (>= 1.40.3), dplyr (>= 0.7.6), stringr (>= 1.3.1), tidyr (>=
        0.8.1), Biostrings (>= 2.48.0), purrr (>= 0.2.5),
        BSgenome.Hsapiens.UCSC.hg38 (>= 1.4.1),
        phastCons100way.UCSC.hg38 (>= 3.7.1), GenomicScores (>= 1.4.1),
        shiny (>= 1.0.5), Gviz (>= 1.24.0), BiocGenerics (>= 0.26.0),
        IRanges (>= 2.14.10), TFBSTools (>= 1.18.0), JASPAR2018 (>=
        1.1.1), tibble (>= 1.4.2), R.utils (>= 2.6.0), stats, utils
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
biocViews: ImmunoOncology, Sequencing, RNASeq, Genetics, Preprocessing,
        Transcription, GeneRegulation
URL: https://github.com/ipumin/primirTSS
BugReports: http://github.com/ipumin/primirTSS/issues
git_url: https://git.bioconductor.org/packages/primirTSS
git_branch: devel
git_last_commit: bbff91f
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-05 00:17:09 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(find_tss)
export(peak_join)
export(peak_merge)
export(plot_primiRNA)
export(run_primirTSSapp)
export(trans_cor)
import(BiocGenerics)
import(GenomicRanges)
import(dplyr, except=c("intersect", "setdiff", "union", "combine"))
importFrom(BSgenome.Hsapiens.UCSC.hg38,Hsapiens)
importFrom(Biostrings,readDNAStringSet)
importFrom(Biostrings,reverseComplement)
importFrom(GenomicScores,gscores)
importFrom(IRanges,IRanges)
importFrom(IRanges,countOverlaps)
importFrom(IRanges,findOverlaps)
importFrom(JASPAR2018,JASPAR2018)
importFrom(R.utils,gunzip)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,subjectHits)
importFrom(phastCons100way.UCSC.hg38,phastCons100way.UCSC.hg38)
importFrom(purrr,pmap)
importFrom(rtracklayer,chrom)
importFrom(rtracklayer,import.chain)
importFrom(rtracklayer,liftOver)
importFrom(stats,runif)
importFrom(stringr,str_detect)
importFrom(stringr,str_remove)
importFrom(stringr,str_split)
importFrom(stringr,str_to_title)
importFrom(tibble,as_tibble)
importFrom(tidyr,separate)
importFrom(tidyr,unite)
importFrom(utils,download.file)

#' primirTSS: Search for putative TSSs of miRNA
#'
#' A fast, convenient tool to identify the TSSs of miRNAs
#' by integrating the data of H3K4me3 and Pol II as well as
#' combining the conservation level and sequence feature,
#' provided within both command-line and graphical interfaces,
#' which achieves a better performance than
#' the previous non-cell-specific methods on miRNA TSSs.
#'
#'
#' @section Detail:
#'   See \code{\link{find_tss}} for deailted instruction
#'   to predict TSSs of miRNA;
#'   See \code{\link{run_primirTSSapp}}
#'   to predict TSSs of miRNA using graphical web interface.
#'
#' @section Author(s):
#'
#'   Maintainer: Pumin Li \email{ipumin@@163.com}
#'
#'   Other contributors: Qi Xu \email{xuqi@@vip.qq.com}
#'
#'
#' @seealso
#'   Useful links:
#'   \href{https://academic.oup.com/bioinformatics/article-lookup/doi/10.1093/bioinformatics/btw171}{Xu HUA et al}
#'
#' @docType package
#' @name primirTSS
#'
NULL

#' Predict TSSs of miRNA using a graphical web interface.
#'
#' A graphical web interface is provided for users to achieve the functions of
#' \code{\link{find_tss}} and \code{\link{plot_primiRNA}} to intuitively and
#' conveniently predict putative TSSs of miRNA.
#'
#' Users can refer documents of the two functions mentioned ABOVE for details.
#'
#' @return A graphical interface.
#'
#' @examples
#'
#' \dontrun{
#' run_primirTSSapp()
#' }
#'
#' @export
run_primirTSSapp <- function() {
  app_path <- system.file("extdata", "primirTSSapp", package = "primirTSS")
  shiny::runApp(app_path)
}

#' Merge adjacent peaks within H3K4me3 or Pol II data.
#'
#' Merge the adjacent segments provided as \code{GRange} object from original
#' data. This function will merge adjacent peaks the distance between which is
#' less than n base pairs apart and then return the merged segments.
#'
#' @param peak A \code{GRange} object. The peaks to be merged from one certain
#'   ChIP-seq data, such as H3K4me3 data or Pol II data.
#' @param n A number. \code{n} stipulates the distance(bp, base pair) between
#'   two seperate peaks within which they should be merged.
#'
#' @return A GRanges object. The merged peaks for the following analysis to
#'   search for TSSs.
#'
#' @examples
#' peak_df <- data.frame(chrom = c("chr1", "chr2", "chr1"),
#'                       chromStart = c(450, 460, 680),
#'                       chromEnd = c(470, 480, 710),
#'                       stringsAsFactors = FALSE)
#' peak <-  as(peak_df, "GRanges")
#'
#' peak_merge(peak, n =250)
#'
#' @export

peak_merge <- function(peak, n = 250) {
  end(peak) <- end(peak) + n
  peak <- reduce(peak)
  end(peak) <- end(peak) - n
  peak
}

#' transform one hg coordinates to another
#'
#'
#' Convert coordinates between different genomes when necessary.
#'
#'
#' @param peak A \code{GRange} object. The genome, the coordinates of which need
#'   to be coverted.
#' @param hg_from The genome are coverting from. This parameter can be "hg18",
#'   "hg19" or "hg38", etc.
#' @param hg_to Which type the genome is converting to. This parameter can be "hg18",
#'   "hg19" or "hg38", etc. NOTICE \code{hg_from} and \code{hg_to} should be
#'   different from each other.
#'
#' @return A GRanges object.
#'
#' @examples
#'
#' peak_df <- data.frame(chrom = c("chr7", "chr7", "chr7"),
#'                       chromStart = c(128043908, 128045075, 128046242),
#'                       chromEnd = c(128045074, 128046241, 128047408),
#'                       stringsAsFactors = FALSE)
#' peak <-  as(peak_df, "GRanges")
#'
#' trans_cor(peak, "hg19", "hg38")
#'
#' @importFrom stringr str_to_title
#' @importFrom R.utils gunzip
#' @importFrom rtracklayer import.chain
#' @importFrom rtracklayer liftOver
#' @importFrom utils download.file
#'
#' @export

trans_cor <- function(peak, hg_from, hg_to) {
  file_name <- sprintf("%sTo%s.over.chain", hg_from, str_to_title(hg_to))
  path <- file.path(system.file(package="primirTSS", "extdata"), file_name)

  gz_name <- file.path(system.file(package="primirTSS", "extdata"),
                       paste0(file_name, ".gz"))

  if (!file.exists(path)) {
    if (!file.exists(gz_name)) {
      url <- sprintf(
        "http://hgdownload.cse.ucsc.edu/goldenPath/%s/liftOver/%s.gz",
        hg_from, file_name)
      .trans_hg_download(url, gz_name)
    }
    gunzip(gz_name)
  }

  path <- system.file(package="primirTSS", "extdata", file_name)
  ch = import.chain(path)
  suppressWarnings(unlist(liftOver(peak, ch)))
}


.trans_hg_download <- function(url, gz_name, download.file, N.TRIES=3L) {
  N.TRIES <- as.integer(N.TRIES)
  stopifnot(length(N.TRIES) == 1L, !is.na(N.TRIES))

  while (N.TRIES > 0L) {
    result <- tryCatch(utils::download.file(url, gz_name), error=identity)
    if (!inherits(result, "error"))
      break
    N.TRIES <- N.TRIES - 1L
  }

  if (N.TRIES == 0L) {
    stop("'trans_cor()' failed:",
         "\n  URL: ", url,
         "\n  error: ", conditionMessage(result))
  }

  result
}

library(shiny)

fluidPage(
  titlePanel("pri-miRNA TSS"),
  navbarPage(
      "",
      tabPanel(
        "Find pri-miRNA TSS",
        sidebarLayout(
          sidebarPanel(
            h3("Find pri-miRNA TSS"),
            br(),

            radioButtons("file1_test", "have histone peaks file",
                         c("YES" = "yes",
                           "NO" = "no"),
                         selected = "yes"),
            fileInput("file1", "choose histone peaks file",
                      multiple = TRUE,
                      accept = c(
                        "text/csv",
                        "text/comma-separated-values,text/plain",
                        ".csv")),
            numericInput(inputId = "n1",
                         label = "merge threshold of histone peaks",
                         value = 250),

            radioButtons("file2_test", "have pol2 peaks files",
                         c("YES" = "yes",
                           "NO" = "no"),
                         selected = "no"),
            fileInput("file2", "choose pol2 peaks files",
                      multiple = TRUE,
                      accept = c(
                        "text/csv",
                        "text/comma-separated-values,text/plain",
                        ".csv")),
            numericInput(inputId = "n2",
                         label = "merge threshold of pol2 peaks",
                         value = 250),

            radioButtons("file3_test", "have miRNA expression profile",
                         c("YES" = "yes",
                           "NO" = "no"),
                         selected = "no"),
            fileInput("file3", "choose expressed miRNAs",
                      multiple = TRUE,
                      accept = c(
                        "text/csv",
                        "text/comma-separated-values,text/plain",
                        ".csv")),

            radioButtons("file4_test", "have DHS file",
                         c("YES" = "yes",
                           "NO" = "no"),
                         selected = "no"),
            fileInput("file4", "choose DHS files",
                      multiple = TRUE,
                      accept = c(
                        "text/csv",
                        "text/comma-separated-values,text/plain",
                        ".csv")),
            radioButtons("DHS_check_result", "allmirdhs_byforce",
                         c("YES" = "yes",
                           "NO" = "no"),
                         selected = "yes"),

            radioButtons("file5_test", "have gene expression profile",
                         c("YES" = "yes",
                           "NO" = "no"),
                         selected = "no"),
            fileInput("file5", "choose expressed genes",
                      multiple = TRUE,
                      accept = c(
                        "text/csv",
                        "text/comma-separated-values,text/plain",
                        ".csv")),
            radioButtons("gene_check_result", "allmirgene_byforce",
                         c("YES" = "yes",
                           "NO" = "no"),
                         selected = "yes"),

            numericInput(inputId = "flanking_num",
                         label = "eponine flanking_num",
                         value = 1000),

            numericInput(inputId = "threshold",
                         label = "eponine threshold",
                         value = 0.7),

            radioButtons("tf", "get TFs simultaneously",
                         c("YES" = "yes",
                           "NO" = "no"),
                         selected = "no"),
            numericInput(inputId = "tf_n",
                         label = "seek n bps upstream from TSSs",
                         value = 1000),
            numericInput(inputId = "min.score",
                         label = "threshold for scoring transcription factor binding sites",
                         value = 0.8),

            actionButton("wm_start", "Start the analysis"),

            downloadButton('tss_result.csv', 'download the result')


          ), # sidebarPanel

          mainPanel(

            h3("View the first six rows of the result"),

            tableOutput("wm_exprs_mx")
          ) # mainPanel

        ), # sidebarLayout

      br(),
      br()
    ),

    tabPanel(
      "Plot pri-miRNA TSS",
      sidebarLayout(
        sidebarPanel(
          h3("plot pri-miRNA TSS"),
          br(),

          radioButtons("file1.2_test", "have histone peaks file",
                       c("YES" = "yes",
                         "NO" = "no"),
                       selected = "yes"),
          fileInput("file1.2", "choose histone peaks files",
                    multiple = TRUE,
                    accept = c(
                      "text/csv",
                      "text/comma-separated-values,text/plain",
                      ".csv")),
          numericInput(inputId = "n1.2",
                       label = "merge threshold of histone peaks",
                       value = 250),

          radioButtons("file2.2_test", "have pol2 peaks files",
                       c("YES" = "yes",
                         "NO" = "no"),
                       selected = "no"),
          fileInput("file2.2", "choose pol2 peaks files",
                    multiple = TRUE,
                    accept = c(
                      "text/csv",
                      "text/comma-separated-values,text/plain",
                      ".csv")),
          numericInput(inputId = "n2.2",
                       label = "merge threshold of pol2 peaks",
                       value = 250),

          textInput("file3.2", "expressed miRNAs",
                    value = "hsa-mir-5697"),

          radioButtons("file4.2_test", "have DHS file",
                       c("YES" = "yes",
                         "NO" = "no"),
                       selected = "no"),
          fileInput("file4.2", "choose DHS files",
                    multiple = TRUE,
                    accept = c(
                      "text/csv",
                      "text/comma-separated-values,text/plain",
                      ".csv")),
          radioButtons("DHS_check_result.2", "allmirdhs_byforce",
                       c("YES" = "yes",
                         "NO" = "no"),
                       selected = "yes"),

          radioButtons("file5.2_test", "have gene expression profile",
                       c("YES" = "yes",
                         "NO" = "no"),
                       selected = "no"),
          fileInput("file5.2", "choose expressed genes",
                    multiple = TRUE,
                    accept = c(
                      "text/csv",
                      "text/comma-separated-values,text/plain",
                      ".csv")),
          radioButtons("gene_check_result.2", "allmirgene_byforce",
                       c("YES" = "yes",
                         "NO" = "no"),
                       selected = "yes"),

          numericInput(inputId = "flanking_num.2",
                       label = "eponine flanking_num",
                       value = 1000),

          numericInput(inputId = "threshold.2",
                       label = "eponine threshold",
                       value = 0.7),

          actionButton("wm_start.2", "Start to plot")

        ), # sidebarPanel

        mainPanel(

          h3("View the plot"),

          plotOutput("wm_plot", width = 1100, height = 700)
        ) # mainPanel

      ), # sidebarLayout

      br(),
      br()
    )


  )
) # fluidPage

#' @importFrom Biostrings readDNAStringSet
#' @importFrom stringr str_remove
#' @importFrom tibble as_tibble
#' @importFrom JASPAR2018 JASPAR2018
#'


mir_tf <- function(mir_name, chrom, strand, predicted_tss,
                   tf_n = 1000, min.score = 0.8) {

  tss_df <- tibble::tibble(mir_name = mir_name,
                       chrom = chrom,
                       strand = strand,
                       predicted_tss = predicted_tss)

  a <- tss_df %>%
    select(mir_name, chrom, strand, predicted_tss)

  a1 <- paste(a$predicted_tss - tf_n, a$predicted_tss, sep = "--")
  a2 <- paste(a$predicted_tss, a$predicted_tss + tf_n, sep = "--")

  region <- ifelse(a$strand == "+", a1, a2)
  tcga <- bind_cols(a, region = region) %>%
    separate(region, into = c("start", "end")) %>%
    select(-predicted_tss) %>%
    mutate(start = as.double(start),
           end = as.double(end)) %>%
    select(chrom, start, end, strand) %>%
    pmap(cor_tdga) %>%
    unlist()

  line1 <- a %>%
    mutate(arrow = ">") %>%
    unite(line1, arrow, mir_name, sep = "")

  a_tmp <- paste(line1$line1, tcga, sep = "\n")
  tmp_path <- file.path(tempdir(), paste0("tf", runif(1, 0, 100), ".fa"))
  writeLines(a_tmp, tmp_path)

  opts <- list()
  opts[["species"]] <- "9606"
  pfms <- TFBSTools::getMatrixSet(JASPAR2018, opts)
  pwms <- TFBSTools::toPWM(pfms)
  fasta_read <- readDNAStringSet(tmp_path)

  file.remove(tmp_path)
  sitesetList <- TFBSTools::searchSeq(pwms, fasta_read, strand="+", min.score)
  a_GFF <- TFBSTools::writeGFF3(sitesetList)
  result <- a_GFF %>%
    separate(attributes,
             into = c("TF", "TF_class", "TF_sequence"),
             sep = ";") %>%
    select(seqname, TF, TF_class)

  result$TF <- str_remove(result$TF, "^TF=")
  result$TF_class <- str_remove(result$TF_class, "^class=")

  s_mir_tf <- function(x) {
    unique(x$TF) %>%
      paste(collapse = ", ")
  }

  s_mir_list <- split(result, result$seqname)
  tf <- lapply(s_mir_list, s_mir_tf) %>% unlist()

  tibble::tibble(mir_name = names(s_mir_list), tf = tf)
}

env_shiny <- new.env()

env_shiny$peak_merge2 <- function(peak, n = 250) {
  peak <- GenomicRanges::makeGRangesFromDataFrame(peak)
  GenomicRanges::end(peak) <- GenomicRanges::end(peak) + n
  peak <- GenomicRanges::reduce(peak)
  GenomicRanges::end(peak) <- GenomicRanges::end(peak) - n
  peak
}

env_shiny$find_tss2 <- function(bed_merged, expressed_mir = "all",
                                flanking_num = 1000, threshold = 0.7,
                                ignore_DHS_check = TRUE,
                                DHS, allmirdhs_byforce = TRUE,
                                expressed_gene = "all", allmirgene_byforce = TRUE,
                                seek_tf = FALSE, tf_n = 1000, min.score = 0.8){
  DHS <- GenomicRanges::makeGRangesFromDataFrame(DHS)
  find_tss(bed_merged, expressed_mir, flanking_num,
           threshold, ignore_DHS_check, DHS = DHS, allmirdhs_byforce,
           expressed_gene, allmirgene_byforce, seek_tf,
           tf_n, min.score)
}

env_shiny$plot_primiRNA2 <- function(expressed_mir, bed_merged,
                                     flanking_num = 1000, threshold = 0.7,
                                     ignore_DHS_check = TRUE, DHS,
                                     allmirdhs_byforce = TRUE,
                                     expressed_gene = "all",
                                     allmirgene_byforce = TRUE){
  DHS <- GenomicRanges::makeGRangesFromDataFrame(DHS)
  plot_primiRNA(expressed_mir, bed_merged, flanking_num, threshold,
                ignore_DHS_check, DHS, allmirdhs_byforce,
                expressed_gene = "all", allmirgene_byforce = TRUE)
}

find_nearest_peak <- function(peak, expressed_mir = "all") {
  `%>%` <- magrittr::`%>%`
  peak <- as.data.frame(peak)
  peak <- peak %>%
    dplyr::mutate(
      chrom = as.character(seqnames),
      start = as.double(start),
      end = as.double(end)
    ) %>%
    dplyr::select(chrom, start, end)

  peak_list <- split(peak, peak$chrom)

  if (length(expressed_mir) > 1 || expressed_mir != "all") {
    id_mir <- mir_loci$mir_name
    mir_loci <- mir_loci[id_mir %in% expressed_mir, ]
  }

  mir_loci_list <- split(mir_loci, mir_loci$chrom)

  all_chr <- intersect(names(mir_loci_list), names(peak_list))

  split_result <- vector("list", length(all_chr))

  for (i in seq_along(all_chr)) {
    tmp_peak <- peak_list[[all_chr[i]]]
    tmp_mir <- mir_loci_list[[all_chr[i]]]

    f_fun <- function(x) {
      compare <- x - tmp_peak$end
      if(all(compare >= 0)) return("*")
      compare[compare >= 0] <- -Inf
      which.max(compare) %>% as.character()
    }
    aa1 <- lapply(tmp_mir$mir_start, f_fun) %>% unlist()

    z_fun <- function(x) {
      compare <- x - tmp_peak$start
      if(all(compare <= 0)) return("*")
      compare[compare <= 0] <- Inf
      which.min(compare) %>% as.character()
    }
    aa2 <- lapply(tmp_mir$mir_start, z_fun) %>% unlist()

    index <- ifelse(tmp_mir$strand == "-", aa1, aa2)

    if (any(index == "*")) {
      rm_index <- which(index == "*")
      tmp_mir <- tmp_mir[-rm_index, ]
      index <- index[-rm_index]
    }

    index <- as.double(index)
    tmp_peak2 <- tmp_peak[index, ]
    colnames(tmp_peak2) <- c("chrom1", "start1", 'end1')
    tmp_df <- dplyr::bind_cols(tmp_mir, tmp_peak2)
    index1 <-
      ((tmp_df$mir_start - tmp_df$start1) * (tmp_df$mir_start - tmp_df$end1)) <=
      0
    tmp_df[(index1 & tmp_df$strand == "+"), ]$end1 <-
      tmp_df[(index1 & tmp_df$strand == "+"), ]$mir_start - 1
    tmp_df[(index1 & tmp_df$strand == "-"), ]$start1 <-
      tmp_df[(index1 & tmp_df$strand == "-"), ]$mir_start + 1

    split_result[[i]] <- tmp_df %>%
      dplyr::select(mir_name, chrom, stem_loop_p1 = start,
                    stem_loop_p2 = end, strand,
                    peak_p1 = start1, peak_p2 = end1)
  }

  success_tf <- Reduce(rbind, split_result)

  fail_res <- setdiff(unique(mir_loci$mir_name), unique(success_tf$mir_name))
  if (length(fail_res) == 0) {
    fail <- "No one miRNA fail to find its nearest peaks"
  } else {
    fail <- fail_res
  }
  list(success = Reduce(rbind, split_result), fail_nearpeak = fail)
}

#' Predict TSSs of miRNA
#'
#' Search for putative TSSs of miRNA, together with integrating available data such
#' as H3K4me3 data, Pol II data, miRNA expression data, and protein-coding gene
#' data, as well as provide the transcriptional regulation relationship between TF
#' and miRNA.
#'
#' @param bed_merged Peaks from ChIP-seq data to be provided for analysis can be
#'   H3K4me3 peaks, Pol II peaks or both. Notice that peaks are supposed to be
#'   merged(see also \code{\link{peak_merge}}) before \code{find_TSS} if using
#'   only one kind of peak data, while peaks should be firstly merged and then
#'   join together(see also \code{\link{peak_join}}) if both H3K4me3 data and
#'   Pol II are input.
#' @param expressed_mir This parameter allows users to specify certain miRNAs,
#'   the TSSs of which they want to search for by providing a list of
#'   miRNAs(e.g., expressed miRNAs in a certain cell-line). If
#'   \code{expressed_mir} is not specified, the default value of the parameter
#'   is "\code{all}" and the function will acquiescently employ all the miRNAs
#'   currently listed on "\code{miRbase}" database.
#' @param flanking_num A parameter in Eponine model to detect TSSs. It is
#'   concluded that a peak signal with flanking regions of C-G enrichment are
#'   important to mark TSSs. The default value is 1000.
#' @param threshold The threshold for candidate TSSs scored with Eponine method. The
#'   default value is 0.7.
#' @param ignore_DHS_check The process of DHS_check further assists to filter
#'   putative TSSs. When there is a DHS peak that locates within 1 kb upstream
#'   of a putative TSS, this predicted TSS will be retained for its character is
#'   consistent with that of an authentic TSS. Or the TSSs with no DHSs locating
#'   within 1 kb upstream of them would be discarded.
#' @param DHS ChIP-seq data of DNase I hypersensitive sites(DHSs).
#' @param allmirdhs_byforce When we use DHS data to check the validity of TSSs,
#'   there is a possibility where no DHSs locates within 1 kb upstream of all
#'   putative TSSs and all these putative TSSs might be filtered out by our
#'   method resulting no outputs. While "\code{allmirdhs_byforce} = TRUE", it
#'   ensures to output at least 1 most possible TSS even if the nearest DHS
#'   signal locates more than 1 kb upstream of this TSS.
#' @param expressed_gene Users can specify genes expressed in certain
#'   cell-lines that are analyzed. Or the default value is "\code{all}", which
#'   means all the expressed genes annotated on Ensemble will be employed.
#' @param allmirgene_byforce While integrating expressed_gene data to improve
#'   prediction, there might be a circumstance where all the putative TSS are
#'   discarded. To prevent this condition, users are allowed to use
#'   "\code{allmirgene_byforce = TRUE}" to ensure at least 1 putative TSS for
#'   each miRNA will be output.
#' @param seek_tf With the result of predicted TSSs, seek_tf provides users with
#'   an option to predict related TFs for miRNA. The data of transcription
#'   factors refer to \code{JASPAR2018} database.
#' @param tf_n TFBS locates on the upstream of the TSS of a certain TF, which is
#'   considered as the promoter region. \code{tf_n} set the length of promoter
#'   region for predicting transcription regulation between miRNAs and TFs.
#' @param min.score The threshold for scoring transcription factor binding sites. A
#'   single absolute value between 0 and 1.
#'
#'
#'
#' @return The first part of the result returns details of predicted TSSs,
#'   composed of seven columns: \code{mir_name, chrom, stem_loop_p1,
#'   stem_loop_p2, strand mir_context, tss_type gene and predicted_tss}:
#'
#'   \code{mir_name}: Name of miRNA.
#'
#'   \code{chrom}: Chromosome.
#'
#'   \code{stem_loop_p1}: The start site of a stem-loop.
#'
#'   \code{stem_loop_p2}: The end site of a stem-loop.
#'
#'   \code{strand}: Polynucleotide strands. (\code{+/-})
#'
#'   \code{mir_context}: The relative positon relationship between stem-loop and
#'   protein-coding gene. (\code{intra/inter})
#'
#'   \code{tss_type}: Four types of predicted TSSs. See the section below
#'   \code{TSS types} for details.
#'   (\code{host_TSS/intra_TSS/overlap_inter_TSS/inter_TSS})
#'
#'   \code{gene}: Ensembl gene ID
#'
#'   \code{predicted_tss}: Predicted transcription start sites(TSSs).
#'
#'   \code{pri_tss_distance}: The distance between a predicted TSS and the start
#'   site of the stem-loop.
#'
#'
#' @section TSS types: TSSs are catalogued into 4 types as below.
#'
#'   \code{host_TSS} The TSSs of miRNA that are close to the TSS of
#'   protein-coding gene implying they may share the same TSS, on the condition
#'   where \code{mir_context} is "\code{intra}". (See above:
#'   \code{Value-mir_context})
#'
#'   \code{intra_TSS} The TSSs of miRNA that are NOT close to the TSS of
#'   the protein-coding gene, on the condition where \code{mir_context} is
#'   "\code{intra}".
#'
#'   \code{overlap_inter_TSS} The TSSs of miRNA are catalogued as
#'   "\code{overlap_inter_TSS}" when the pri-miRNA gene overlaps with Ensembl
#'   gene, on the condition where "\code{mir_context}" is "\code{inter}".
#'
#'   \code{inter_inter_TSS} The TSSs of miRNA are catalogued as
#'   "\code{inter_inter_TSS}" when the miRNA gene does NOT overlap with Ensembl
#'   gene, on the condition where "\code{mir_context}" is "\code{inter}".
#'
#'   (See Xu HUA et al 2016 for more details)
#'
#'
#'
#' @section Log:
#'
#'   The second part of the result returns logs during the process of
#'   prediction: \code{find_nearest_peak_log} If no peaks locate in the
#'   upstream of a stem-loop to help determine putative TSSs of miRNA, we will
#'   fail to find the nearest peak and this miRNA will be logged in
#'   \code{find_nearest_peak_log}.
#'
#'   \code{eponine_score_log} For a certain miRNA, if none of the candidate
#'   TSSs scored with \code{Eponine method} meet the threshold we set, we will
#'   fail to get an eponine score and this miRNA will be logged in
#'   \code{eponine_score_log}.
#'
#'   \code{DHS_check_log} For a certain miRNA, if no DHS signals locate
#'   within 1 kb upstream of each putative TSSs, these putative TSSs will be
#'   filtered out and this miRNA will be logged in \code{DHS_check_log}.
#'
#'   \code{gene_filter_log} For a certain miRNA, when integrating
#'   expressed_gene data to improve prediction, if no putative TSSs are
#'   confirmed after considering the relative position relationship among TSSs,
#'   stem-loops and expressed genes, this miRNA will be filtered out and logged
#'   in \code{gene_filter_log}.
#'
#' @section Reference: Xu Hua, Luxiao Chen, Jin Wang*, Jie Li* and Edgar
#'   Wingender*, Identifying cell-specific microRNA transcriptional start sites.
#'   Bioinformatics 2016, 32(16), 2403-10.
#'
#'
#' @examples
#'
#' bed_merged <- data.frame(
#'                 chrom = c("chr1", "chr1", "chr1", "chr1", "chr2"),
#'                 start = c(9910686, 9942202, 9996940, 10032962, 9830615),
#'                 end = c(9911113, 9944469, 9998065, 10035458, 9917994),
#'                 stringsAsFactors = FALSE)
#' bed_merged <- as(bed_merged, "GRanges")
#'
#' \dontrun{
#' ownmiRNA <- find_tss(bed_merged, expressed_mir = "hsa-mir-5697",
#'                      ignore_DHS_check = TRUE,
#'                      expressed_gene = "all",
#'                      allmirgene_byforce = TRUE)
#' }
#'
#' @export
find_tss <- function(bed_merged, expressed_mir = "all",
                     flanking_num = 1000, threshold = 0.7,
                     ignore_DHS_check = TRUE,
                     DHS, allmirdhs_byforce = TRUE,
                     expressed_gene = "all", allmirgene_byforce = TRUE,
                     seek_tf = FALSE, tf_n = 1000, min.score = 0.8) {

  h3_mir_flank <- find_nearest_peak(bed_merged, expressed_mir)

  mir_eponine_score <- eponine_score(h3_mir_flank$success$mir_name,
                                     h3_mir_flank$success$chrom,
                                     h3_mir_flank$success$stem_loop_p1,
                                     h3_mir_flank$success$stem_loop_p2,
                                     h3_mir_flank$success$strand,
                                     h3_mir_flank$success$peak_p1,
                                     h3_mir_flank$success$peak_p2,
                                     flanking_num, threshold)

  candidate_tss <- find_candidate_tss(mir_eponine_score$success$mir_name,
                                      mir_eponine_score$success$chrom,
                                      mir_eponine_score$success$stem_loop_p1,
                                      mir_eponine_score$success$stem_loop_p2,
                                      mir_eponine_score$success$strand,
                                      mir_eponine_score$success$tss_p1,
                                      mir_eponine_score$success$tss_p2,
                                      mir_eponine_score$success$eponine_score,
                                      ignore_DHS_check,
                                      DHS, allmirdhs_byforce)

  gene_fliter_tss <- tss_filter(candidate_tss$mir_df$mir_name,
                                candidate_tss$mir_df$chrom,
                                candidate_tss$mir_df$stem_loop_p1,
                                candidate_tss$mir_df$stem_loop_p2,
                                candidate_tss$mir_df$strand,
                                candidate_tss$mir_df$tss_p1,
                                candidate_tss$mir_df$tss_p2,
                                expressed_gene, allmirgene_byforce)

  if (any(h3_mir_flank$fail_nearpeak !=
          "No one miRNA fail to find its nearest peaks")) {
    if (length(h3_mir_flank$fail_nearpeak) == 1) {
      log_1 <- paste(h3_mir_flank$fail_nearpeak,
                     "fail to find its nearest peaks", " ")
    } else {
      log_1 <- paste(h3_mir_flank$fail_nearpeak, collapse = ", ")
      log_1 <- paste(log_1, "fail to find their nearest peaks", " ")
    }
  } else {
    log_1 <- h3_mir_flank$fail_nearpeak
  }

  if (any(mir_eponine_score$fail_eponine != "All miRNAs have eponine socres")) {
    if (length(mir_eponine_score$fail_eponine) == 1) {
      log_2 <- paste(mir_eponine_score$fail_eponine,
                     "fail to have eponine socres", " ")
    } else {
      log_2 <- paste(mir_eponine_score$fail_eponine, collapse = ", ")
      log_2 <- paste(log_2, "fail to have eponine socres", " ")
    }
  } else {
    log_2 <- mir_eponine_score$fail_eponine
  }

  if (any(candidate_tss$fail_DHS != "Do not have a DHS check") &&
      any(candidate_tss$fail_DHS != "All miRNA have DHS verified")) {
    if (length(candidate_tss$fail_DHS) == 1) {
      log_3 <- paste(candidate_tss$fail_DHS, "fail to have a DHS check", " ")
    } else {
      log_3 <- paste(candidate_tss$fail_DHS, collapse = ", ")
      log_3 <- paste(log_3, "fail to have a DHS check", " ")
    }
  } else {
    log_3 <- candidate_tss$fail_DHS
  }

  if (any(gene_fliter_tss$fail_genefilter != "All miRNAs have tss")) {
    if (length(gene_fliter_tss$fail_genefilter) == 1) {
      log_4 <- paste(gene_fliter_tss$fail_genefilter, "fail to have a tss", " ")
    } else {
      log_4 <- paste(gene_fliter_tss$fail_genefilter, collapse = ", ")
      log_4 <- paste(log_4, "fail to have a tss", " ")
    }
  } else {
    log_4 <- gene_fliter_tss$fail_genefilter
  }

  if (allmirgene_byforce == TRUE) {
    log_4 <- "All miRNAs have tss"
  }

  if (seek_tf == TRUE) {
    a <- mir_tf(gene_fliter_tss$tss_df$mir_name, gene_fliter_tss$tss_df$chrom,
                gene_fliter_tss$tss_df$strand,
                gene_fliter_tss$tss_df$predicted_tss,
                tf_n, min.score)

    result <- dplyr::left_join(gene_fliter_tss$tss_df, a, by = "mir_name")
  } else {
    result <- gene_fliter_tss$tss_df
  }

  list(tss_df = result,
       find_nearest_peak_log = log_1,
       eponine_score_log = log_2,
       DHS_check_log = log_3,
       gene_filter_log = log_4)
}

plot_primiRNA_track <- function(expressed_mir, bed_merged,
                          flanking_num = 1000, threshold = 0.7,
                          ignore_DHS_check = TRUE,
                          DHS, allmirdhs_byforce = TRUE,
                          expressed_gene = "all",
                          allmirgene_byforce = TRUE) {

  genome_version <- "hg38"
  mir_peaks <- find_nearest_peak(bed_merged, expressed_mir)
  mir_eponine_score <- eponine_score(mir_peaks$success$mir_name,
                                     mir_peaks$success$chrom,
                                     mir_peaks$success$stem_loop_p1,
                                     mir_peaks$success$stem_loop_p2,
                                     mir_peaks$success$strand,
                                     mir_peaks$success$peak_p1,
                                     mir_peaks$success$peak_p2,
                                     flanking_num, threshold)

  candidate_tss <- find_candidate_tss(mir_eponine_score$success$mir_name,
                                      mir_eponine_score$success$chrom,
                                      mir_eponine_score$success$stem_loop_p1,
                                      mir_eponine_score$success$stem_loop_p2,
                                      mir_eponine_score$success$strand,
                                      mir_eponine_score$success$tss_p1,
                                      mir_eponine_score$success$tss_p2,
                                      mir_eponine_score$success$eponine_score,
                                      ignore_DHS_check,
                                      DHS, allmirdhs_byforce)

  tss_list <- tss_filter(candidate_tss$mir_df$mir_name,
                         candidate_tss$mir_df$chrom,
                         candidate_tss$mir_df$stem_loop_p1,
                         candidate_tss$mir_df$stem_loop_p2,
                         candidate_tss$mir_df$strand,
                         candidate_tss$mir_df$tss_p1,
                         candidate_tss$mir_df$tss_p2,
                         expressed_gene, allmirgene_byforce)

  tss <- tss_list$tss_df

  ep_con <- find_candidate_tss_plot(mir_eponine_score$success$mir_name,
                                    mir_eponine_score$success$chrom,
                                    mir_eponine_score$success$stem_loop_p1,
                                    mir_eponine_score$success$stem_loop_p2,
                                    mir_eponine_score$success$strand,
                                    mir_eponine_score$success$tss_p1,
                                    mir_eponine_score$success$tss_p2,
                                    mir_eponine_score$success$eponine_score,
                                    ignore_DHS_check,
                                    DHS, allmirdhs_byforce)

  tss_tmp <- tss %>%
    mutate(start = predicted_tss,
           end = predicted_tss + 1,
           symbol_name = "tss") %>%
    select(chrom, start, end, strand, symbol_name)
  tss_p <- tss %>%
    mutate(symbol_name = "stem-loop") %>%
    select(chrom, start = stem_loop_p1, end = stem_loop_p2, strand, symbol_name) %>%
    bind_rows(tss_tmp)

  chr <- unique(tss_p$chrom)
  tsstrack <- Gviz::GeneRegionTrack(tss_p, genome = genome_version,
                             chromosome = chr, name = "pri-miRNA",
                             showId = TRUE,
                             symbol = tss_p$symbol_name,
                             just.group = "above",
                             showOverplotting = FALSE)

  gene_p <- gene_loci[gene_loci$gene_id == tss$gene, ] %>%
    mutate(symbol_name = gene_id) %>%
    select(start = gene_p1, end = gene_p2, strand, symbol_name)
  genetrack <- Gviz::GeneRegionTrack(gene_p, genome = genome_version,
                               chromosome = chr, name = "Ensemble genes",
                               shape = "arrow",
                               collapseTranscripts = "meta",
                               just.group = "above",
                               showId = TRUE,
                               showOverplotting = FALSE,
                               symbol = gene_p$symbol_name)

  eponine_p <- ep_con %>%
    mutate(start = tss_p1,
           end = tss_p2) %>%
    select(start, end, strand, data = eponine_score)

  e_track = Gviz::DataTrack(range = eponine_p, genome = genome_version,
                      chromosome = chr,
                      name = "eponine score",
                      type = "h",
                      col.line = "skyblue",
                      ylim = c(0, 1.2))

  con_p <- ep_con %>%
    mutate(start = tss_p1,
           end = tss_p2) %>%
    select(start, end, strand, data = phast_score)

  con_track = Gviz::DataTrack(range = con_p, genome = genome_version,
                      chromosome = chr,
                      name = "conservation score",
                      type = "h",
                      col.line = "skyblue",
                      ylim = c(0, 1.2))

  min_loci <- min(gene_p$start, gene_p$end, tss_p$start, tss_p$end)
  max_loci <- max(gene_p$start, gene_p$end, tss_p$start, tss_p$end)

  list(tsstrack = tsstrack,
       genetrack = genetrack,
       e_track = e_track,
       con_track = con_track,
       min_loci = min_loci,
       max_loci= max_loci,
       chr = chr)
}


#' Plot the result of prediction for miRNA
#'
#' For each miRNA, plot the position of TSS, pri-miRNA, related Ensemble gene,
#' eponine socre and conservation score according to the result of prediction
#' using primirTSS.
#'
#' NOTICE that this function is used for visualizing the predicted result of
#' ONLY ONE specific miRNA every single time.
#'
#' @param expressed_mir This parameter allows users to specify certain miRNAs,
#'   the TSSs of which they want to search for by providing a list of
#'   miRNAs(e.g. expressed miRNAs in a certain cell-line). If
#'   \code{expressed_mir} is not specified, the default value of the parameter
#'   is "\code{all}" and the function will acquiescently employ all the miRNAs
#'   currently listed on "\code{miRbase}" database.
#' @param bed_merged Peaks from ChIP-seq data to be provided for analysis can be
#'   H3K4me3 peaks, Pol II peaks or both. Notice that peaks are supposed to be
#'   merged(see also \code{\link{peak_merge}}) before \code{find_TSS} if using
#'   only one kind of peak data, while peaks shoud be firstly merged and then
#'   join together(see also \code{\link{peak_join}}) if both H3K4me3 data and
#'   Pol II are input.
#' @param flanking_num A parameter in Eponine model to detect TSSs. It is
#'   concluded that a peak signal with flanking regions of C-G enrichment are
#'   important to mark TSSs. The default value is 1000.
#' @param threshold Threshold for candidate TSSs scored with Eponine method. The
#'   default value is 0.7.
#' @param ignore_DHS_check The process of DHS_check further assist to filter
#'   putative TSSs. When there are a DHS peak that locates within 1 kb upstream
#'   of a putative TSS, this predicted TSS will be retain for it character is
#'   consistent with that of an authentic TSS. Or the TSSs with no DHSs locating
#'   within 1 kb upstream of them would be discard.
#' @param DHS ChIP-seq data of DNase I hypersensitive sites(DHSs).
#' @param allmirdhs_byforce When we use DHS data to check the validity of TSSs,
#'   there is possibility where no DHSs locates within 1 kb upstream of all
#'   putative TSSs and all these putative TSSs might be filtered out by our
#'   method resulting no outputs. While "\code{allmirdhs_byforce} = TRUE", it
#'   ensures to output at least 1 most possible TSS even if the nearest DHS
#'   signal locates more than 1 kb upsteam of this TSS.
#' @param expressed_gene Users can speicify genes expressed in certain
#'   cell-lines that is analyzed. Or the default value is "\code{all}", which
#'   means all the expressed genes annotated on Ensemble will be employed.
#' @param allmirgene_byforce While integrating expressed_gene data to improve
#'   prediction, there might be a circumstance where all the putative TSS are
#'   discarded. To prevent this condition, users are allowed to use
#'   "\code{allmirgene_byforce = TRUE}" to ensure at least 1 putative TSS for
#'   each miRNA will be output.
#'
#' @return There will be six tracks plotted as return:
#'
#' \code{Chrom}: Position of miRNA on the chromosome.
#'
#' \code{hg38}: Reference genome coordinate in hg38.
#'
#' \code{pri-miRNA}: Position of pri-miRNA.
#'
#' \code{Ensemble genes}: Position of related protein-coding gene.
#'
#' \code{eponine score}: Score of best putative TSS conducted by eponine method.
#'
#' \code{conservation score}: Conservation score should be integrated with
#' eponine score to find out putative TSSs.
#'
#'
#'
#'
#' @examples
#' expressed_mir <- "hsa-mir-5697"
#' bed_merged <- data.frame(
#'   chrom = c("chr1", "chr1", "chr1", "chr1", "chr2"),
#'   start = c(9180799, 9201483, 9234339, 9942202, 9830615),
#'   end = c(9183889, 9202580, 9235853, 9944469, 9917994),
#'   stringsAsFactors = FALSE
#' )
#' bed_merged <- as(bed_merged, "GRanges")
#' \dontrun{
#' plot_primiRNA(expressed_mir, bed_merged)
#' }
#'
#' @importFrom rtracklayer chrom
#' @export

plot_primiRNA <- function(expressed_mir, bed_merged,
                          flanking_num = 1000, threshold = 0.7,
                          ignore_DHS_check = TRUE,
                          DHS, allmirdhs_byforce = TRUE,
                          expressed_gene = "all",
                          allmirgene_byforce = TRUE) {

  track <- plot_primiRNA_track(expressed_mir, bed_merged,
                               flanking_num, threshold,
                               ignore_DHS_check,
                               DHS, allmirdhs_byforce,
                               expressed_gene,
                               allmirgene_byforce)

  genome_version <- "hg38"
  itrack <- Gviz::IdeogramTrack(genome = genome_version, showBandId = TRUE, name = "")
  axistrack <- Gviz::GenomeAxisTrack(add53=TRUE, exponent=0,
                               littleTicks=TRUE, name=genome_version)

  tsstrack <- track$tsstrack
  genetrack <- track$genetrack
  e_track <- track$e_track
  con_track <- track$con_track
  chr <- track$chr
  START <- track$min_loci
  END <- track$max_loci

  Gviz::plotTracks(
    list(itrack,
         axistrack,
         tsstrack,
         genetrack,
         e_track,
         con_track),
    chromosome = chr,
    from = START,
    to = END,
    extend.left = 0.1,
    extend.right = 0.1,
    fontcolor.group = "black",
    fontcolor = "black",
    background.title = "darkblue",
    col = NULL,
    showTitle = TRUE,
    frame = TRUE,
    fontsize = 11,
    fontsize.group = 11,
    cex = 0.8,
    cex.group = 0.8,
    cex.title = 1,
    sizes = c(1.5, 1.5, 1.5, 1.5, 3, 3)
  )
}

#' @importFrom phastCons100way.UCSC.hg38 phastCons100way.UCSC.hg38
#' @importFrom GenomicScores gscores
#' @rawNamespace import(dplyr, except=c("intersect", "setdiff", "union", "combine"))
#' @import BiocGenerics
#' @import GenomicRanges


phast_score <- function(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                        strand, tss_p1, tss_p2, eponine_score) {

  mir_eponine_score <- tibble::tibble(mir_name = mir_name, chrom = chrom,
                                  stem_loop_p1 = stem_loop_p1,
                                  stem_loop_p2 = stem_loop_p2,
                                  strand = strand, tss_p1 = tss_p1,
                                  tss_p2 = tss_p2,
                                  eponine_score = eponine_score)

  phast <- phastCons100way.UCSC.hg38

  score_s <- function(loci) {
    gscores(phast, GRanges(loci))$default
  }

  mir_eponine_score %>%
    mutate(loci = paste(chrom, paste(tss_p1, tss_p2, sep = "-"), sep = ":"),
           phast_score = score_s(loci)) %>%
    group_by(mir_name) %>%
    mutate(eponine_rank = rank(eponine_score),
           phast_rank = rank(phast_score),
           e_p_rank = eponine_rank + phast_rank) %>%
    filter(e_p_rank == max(e_p_rank)) %>%
    ungroup() %>%
    select(-(loci:e_p_rank), -eponine_score)
}

phast_score_plot <- function(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                             strand, tss_p1, tss_p2, eponine_score) {

  mir_eponine_score <- tibble::tibble(mir_name = mir_name, chrom = chrom,
                                  stem_loop_p1 = stem_loop_p1,
                                  stem_loop_p2 = stem_loop_p2,
                                  strand = strand, tss_p1 = tss_p1,
                                  tss_p2 = tss_p2,
                                  eponine_score = eponine_score)

  phast <- phastCons100way.UCSC.hg38

  score_s <- function(loci) {
    gscores(phast, GRanges(loci))$default
  }

  mir_eponine_score %>%
    mutate(loci = paste(chrom, paste(tss_p1, tss_p2, sep = "-"), sep = ":"),
           phast_score = score_s(loci))
}



#' @importFrom IRanges IRanges
#' @importFrom IRanges countOverlaps
#' @importFrom IRanges findOverlaps
#' @importFrom S4Vectors queryHits
#' @importFrom S4Vectors subjectHits
#'
check_DHS_s <- function(chrom, strand, tss_p1, tss_p2, DHS) {

  DHS <- as.data.frame(DHS)
  DHS <- DHS %>%
    mutate(chrom = as.character(seqnames),
           dhs_p1 = as.double(start),
           dhs_p2 = as.double(end)) %>%
    select(chrom, dhs_p1, dhs_p2)

  if (strand == "+") {
    DHS_d <- DHS %>%
      mutate(dhs_p1_down = dhs_p1,
             dhs_p2_down = dhs_p2 + 1000)
  } else {
    DHS_d <- DHS %>%
      mutate(dhs_p1_down = dhs_p1 - 1000,
             dhs_p2_down = dhs_p2)
  }


  DHS_dc <- DHS_d[DHS_d$chrom == chrom, ]

  ir1 <- IRanges(start = tss_p1, end = tss_p2)
  ir2 <- IRanges(start = DHS_dc$dhs_p1_down, end = DHS_dc$dhs_p2_down)

  if (countOverlaps(ir1, ir2) != 0) {
    ov <- findOverlaps(ir1, ir2)
    tmp <- IRanges::intersect(ir1[queryHits(ov)], ir2[subjectHits(ov)])
    result <- paste(paste(start(tmp), end(tmp), sep = "_"), collapse = ":")
  } else {
    result <- "*"
  }

  result
}

check_DHS_df <- function(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                         strand, tss_p1, tss_p2, eponine_score, DHS) {

  x <- tibble::tibble(mir_name = mir_name, chrom = chrom,
                  stem_loop_p1 = stem_loop_p1,
                  stem_loop_p2 = stem_loop_p2,
                  strand = strand, tss_p1 = tss_p1,
                  tss_p2 = tss_p2,
                  eponine_score = eponine_score)
  candidate_tss <- x %>%
    select(chrom, strand, tss_p1, tss_p2) %>%
    pmap(check_DHS_s, DHS) %>%
    unlist()

  new_mir_tss_df1 <- x %>%
    bind_cols(can_tss = candidate_tss) %>%
    filter(can_tss != "*") %>%
    extand_df("can_tss", ":") %>%
    select(-tss_p1, -tss_p2) %>%
    separate(new_info, into = c("tss_p1", "tss_p2"), sep = "_") %>%
    mutate(tss_p1 = as.double(tss_p1),
           tss_p2 = as.double(tss_p2))

  new_mir_tss_df2 <- x %>%
    bind_cols(can_tss = candidate_tss)

  f2 <- function(x) {
    if (all(unique(x$can_tss) == "*")) {
      unique(x$mir_name)
    } else {
      NULL
    }
  }

  DHS_fail_mir <- lapply(split(new_mir_tss_df2, new_mir_tss_df2$mir_name),
                         f2) %>% unlist()

  list(success = new_mir_tss_df1, fail_DHS = DHS_fail_mir)
}


find_candidate_tss <- function(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                               strand, tss_p1, tss_p2, eponine_score,
                               ignore_DHS_check = TRUE,
                               DHS, allmirdhs_byforce = TRUE) {

  mir_eponine_score <- tibble::tibble(mir_name = mir_name, chrom = chrom,
                                  stem_loop_p1 = stem_loop_p1,
                                  stem_loop_p2 = stem_loop_p2,
                                  strand = strand, tss_p1 = tss_p1,
                                  tss_p2 = tss_p2,
                                  eponine_score = eponine_score)

  if (ignore_DHS_check == TRUE) {

    a <- phast_score(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                     strand, tss_p1, tss_p2, eponine_score)
    fail <- "Do not have a DHS check"
  } else {

    dhs_check <- check_DHS_df(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                              strand, tss_p1, tss_p2, eponine_score, DHS)

    mir_eponine_score_s <- dhs_check$success

    a <- phast_score(mir_eponine_score_s$mir_name, mir_eponine_score_s$chrom,
                     mir_eponine_score_s$stem_loop_p1,
                     mir_eponine_score_s$stem_loop_p2,
                     mir_eponine_score_s$strand,
                     mir_eponine_score_s$tss_p1, mir_eponine_score_s$tss_p2,
                     mir_eponine_score_s$eponine_score)

    if (is.null(dhs_check$fail_DHS)) {
      fail <- "All miRNA have DHS verified"
    } else {
      fail <- dhs_check$fail_DHS
    }

    if (allmirdhs_byforce == TRUE && !is.null(dhs_check$fail_DHS)) {
      new_mir_tss_df2 <- mir_eponine_score %>%
        filter(mir_name %in% dhs_check$fail_DHS)
      b <- phast_score(new_mir_tss_df2$mir_name, new_mir_tss_df2$chrom,
                       new_mir_tss_df2$stem_loop_p1,
                       new_mir_tss_df2$stem_loop_p2, new_mir_tss_df2$strand,
                       new_mir_tss_df2$tss_p1, new_mir_tss_df2$tss_p2,
                       new_mir_tss_df2$eponine_score)
      a <- bind_rows(a, b)
    }
  }

  list(mir_df = a, fail_DHS = fail)
}



find_candidate_tss_plot <- function(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                                    strand, tss_p1, tss_p2, eponine_score,
                                    ignore_DHS_check = TRUE,
                                    DHS, allmirdhs_byforce = TRUE) {

  mir_eponine_score <- tibble::tibble(mir_name = mir_name, chrom = chrom,
                                  stem_loop_p1 = stem_loop_p1,
                                  stem_loop_p2 = stem_loop_p2,
                                  strand = strand, tss_p1 = tss_p1,
                                  tss_p2 = tss_p2,
                                  eponine_score = eponine_score)


  if (ignore_DHS_check == TRUE) {
    a <- phast_score_plot(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                          strand, tss_p1, tss_p2, eponine_score)

  } else {

    dhs_check <- check_DHS_df(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                              strand, tss_p1, tss_p2, eponine_score, DHS)

    if (is.null(dhs_check$fail_DHS)) {
      a <- phast_score_plot(dhs_check$success$mir_name, dhs_check$success$chrom,
                            dhs_check$success$stem_loop_p1,
                            dhs_check$success$stem_loop_p2,
                            dhs_check$success$strand,
                            dhs_check$success$tss_p1, dhs_check$success$tss_p2,
                            dhs_check$success$eponine_score)
    } else if (allmirdhs_byforce == TRUE) {
      a <- phast_score_plot(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                            strand, tss_p1, tss_p2, eponine_score)
    } else{
      stop("The miRNA you selected failed to pass the DHS check.
           If you want to have the plot anyway,
           you should set the parameter 'allmirdhs_byforce = TRUE'")
    }
  }
  a
}

#' Integrate H3K4me3 data and Pol II data.
#'
#' Integrate peaks from H3K4me3 and Pol II data. To conduct the overlappd ranges
#' for the further analysis by imposing H3K4me3 peaks on Pol II peaks, if both
#' of these two different kinds of ChIP-seq data are available.
#'
#' @param peak1 H3K4me3 peaks. Merged peak data as \code{GRange} object by
#'   fucntion \code{peak_merge}
#' @param peak2 Pol II peaks. Merged peak data as \code{GRange} object by
#'   fucntion \code{peak_merge}
#'
#' @section Detail: Peak1 and peak2 are signals seperately from the ChIP-seq
#'   data of H3K4me3 and Pol II data that to be integrated. The data is
#'   \code{GRange} object containing three columns \code{Chrom}, \code{Ranges},
#'   \code{Strand}. And the order of these two kinds of data when input as
#'   \code{peak1} and \code{peak2} can be swapped.
#'
#' @return A GRanges object. The joined peaks for the following analysis to
#'   search for TSSs.
#'
#' @examples
#' peak_df1 <- data.frame(chrom = c("chr1", "chr1", "chr1", "chr2"),
#'                        start = c(100, 460, 600, 70),
#'                        end = c(200, 500, 630, 100),
#'                        stringsAsFactors = FALSE)
#' peak1 <-  as(peak_df1, "GRanges")
#'
#' peak_df2 <- data.frame(chrom = c("chr1", "chr1", "chr1", "chr2"),
#'                        start = c(160, 470, 640, 71),
#'                        end = c(210, 480, 700, 90),
#'                        stringsAsFactors = FALSE)
#' peak2 <-  as(peak_df2, "GRanges")
#'
#' peak_join(peak1, peak2)
#'
#' @importFrom S4Vectors queryHits
#' @importFrom S4Vectors subjectHits
#' @export

peak_join <- function(peak1, peak2) {
  ov <- findOverlaps(peak1, peak2)
  intersect(peak1[queryHits(ov)], peak2[subjectHits(ov)])
}

tss_single_judge2 <- function(stem_loop_p1, stem_loop_p2, chrom, strand, tss_p1,
                              tss_p2, gene_loci) {

  gene_df <- gene_loci[gene_loci$chrom == chrom & gene_loci$strand == strand, ]

  intra_index <- stem_loop_p1 >= gene_df$gene_p1 & stem_loop_p2 <=
    gene_df$gene_p2
  if (any(intra_index)) {
    gene_intra <- gene_df[intra_index, ]
    closest_tss <- lapply(gene_intra$gene_start, function(x) {
      min(abs(x - tss_p1), abs(x - tss_p2))}) %>%
      unlist()
    in_tss <- (gene_intra$gene_start - tss_p1) *
      (gene_intra$gene_start - tss_p2) < 0
    colsest_index <- closest_tss <= 150 | in_tss
    if (any(colsest_index)) {
      result <- paste("intra__host_TSS",
                      gene_intra$gene_id[colsest_index],
                      gene_intra$gene_start[colsest_index],
                      # closest_tss[colsest_index],
                      sep = "__") %>%
        paste0(collapse = ":")
    } else {
      tss_site <- ifelse(strand == "+", tss_p2, tss_p1)
      both_site_index <- (tss_site - gene_intra$gene_p1) *
        (tss_site - gene_intra$gene_p2) < 0
      if (any(both_site_index)) {
        gene_intra_2 <- gene_intra[both_site_index, ]
        distance <- abs(tss_site - gene_intra_2$gene_start)
        gene_intra_2_index <- which.min(distance)
        result <- paste("intra__intra_TSS",
                        gene_intra_2$gene_id[gene_intra_2_index],
                        tss_site,
                        sep = "__")
      } else {
        distance <- abs(tss_site - gene_intra$gene_start)
        gene_intra_2_index <- which.min(distance)
        result <- paste("intra__intra_TSS2",
                        gene_intra$gene_id[gene_intra_2_index],
                        tss_site,
                        sep = "__")
      }
    }
  } else {
    closest_tss <- lapply(gene_df$gene_start, function(x) {
      min(abs(x - tss_p1), abs(x - tss_p2))
    }) %>%
      unlist()
    in_tss <- (gene_df$gene_start - tss_p1) * (gene_df$gene_start - tss_p2) < 0
    colsest_index <- closest_tss <= 150 | in_tss

    if (any(colsest_index)) {
      tss_site <- ifelse(strand == "+", tss_p2, tss_p1)
      mir_end <- ifelse(strand == "+", stem_loop_p2, stem_loop_p1)

      distance <- abs(tss_site - gene_df$gene_start)
      gene_inter_index <- which.min(distance)
      result <- paste("inter__overlap_inter_TSS2",
                      gene_df$gene_id[gene_inter_index],
                      tss_site,
                      sep = "__")
    } else {
      tss_site <- ifelse(strand == "+", tss_p2, tss_p1)
      mir_end <- ifelse(strand == "+", stem_loop_p2, stem_loop_p1)
      gene_tss_trans_index <- (tss_site - gene_df$gene_start) *
        (mir_end - gene_df$gene_start) < 0

      if (any(gene_tss_trans_index)) {
        distance <- abs(tss_site - gene_df$gene_start)
        gene_inter_index <- which.min(distance)
        result <- paste("inter__overlap_inter_TSS2",
                        gene_df$gene_id[gene_inter_index],
                        tss_site,
                        sep = "__")
      } else {
        gene_end <- ifelse(gene_df$strand == "+", gene_df$gene_p2,
                           gene_df$gene_p1)
        gene_end_trans_index <- (tss_site - gene_end) * (mir_end - gene_end) < 0
        if (any(gene_end_trans_index)) {
          select_gene <- gene_df[gene_end_trans_index, ]
          distance <- abs(tss_site - select_gene$gene_start)
          gene_inter_index <- which.min(distance)
          result <- paste("inter__overlap_inter_TSS",
                          select_gene$gene_id[gene_inter_index],
                          tss_site,
                          sep = "__")
        } else {
          distance <- abs(tss_site - gene_df$gene_start)
          gene_inter_index <- which.min(distance)
          result <- paste("inter__inter_TSS",
                          gene_df$gene_id[gene_inter_index],
                          tss_site,
                          sep = "__")
        }
      }
    }
  }
  result
}


tss_filter <- function(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                       strand, tss_p1, tss_p2, expressed_gene = "all",
                       allmirgene_byforce = TRUE) {

  candidate_tss <- tibble::tibble(mir_name = mir_name, chrom = chrom,
                              stem_loop_p1 = stem_loop_p1,
                              stem_loop_p2 = stem_loop_p2,
                              strand = strand, tss_p1 = tss_p1,
                              tss_p2 = tss_p2)

  if (length(expressed_gene) > 1 || expressed_gene != "all") {
    gene_loci <- gene_loci %>%
      filter(gene_id %in% expressed_gene)
  }

  tss_info <- candidate_tss %>%
    select(stem_loop_p1, stem_loop_p2, chrom, strand, tss_p1, tss_p2) %>%
    pmap(tss_single_judge2, gene_loci) %>%
    unlist()

  all_mir_tss <- candidate_tss %>%
    bind_cols(tss_info = tss_info)

  s_tss <- all_mir_tss %>%
    select(-tss_p1, -tss_p2) %>%
    extand_df("tss_info", ":") %>%
    separate(new_info,
             into = c("mir_context", "tss_type", "gene", "predicted_tss"),
             sep = "__") %>%
    mutate(predicted_tss = as.double(predicted_tss))

  s_tss_success <- s_tss %>%
    filter(tss_type %in%
             c("host_TSS", "intra_TSS", "overlap_inter_TSS", "inter_TSS"))
  a1 <- s_tss_success$stem_loop_p1 - s_tss_success$predicted_tss
  a2 <- s_tss_success$predicted_tss - s_tss_success$stem_loop_p2
  pri_tss_distance <- ifelse(s_tss_success$strand == "+", a1, a2)

  result <- bind_cols(s_tss_success, pri_tss_distance = pri_tss_distance) %>%
    group_by(mir_name) %>%
    filter(pri_tss_distance == min(pri_tss_distance)) %>%
    ungroup()

  fail_mir <- setdiff(unique(candidate_tss$mir_name), unique(result$mir_name))

  if (length(fail_mir) == 0) {
    fail <- "All miRNAs have tss"
  } else {
    fail <- fail_mir
  }

  if (allmirgene_byforce == TRUE && length(fail_mir) != 0) {
    a_tmp <- s_tss %>%
      filter(mir_name %in% fail)
    aa1 <- a_tmp$stem_loop_p1 - a_tmp$predicted_tss
    aa2 <- a_tmp$predicted_tss - a_tmp$stem_loop_p2
    pri_tss_distance2 <- ifelse(a_tmp$strand == "+", aa1, aa2)
    fail_res <- bind_cols(a_tmp, pri_tss_distance = pri_tss_distance2) %>%
      group_by(mir_name) %>%
      filter(pri_tss_distance == min(pri_tss_distance)) %>%
      ungroup()

    fail_res$tss_type <- str_remove(fail_res$tss_type, "2")

    result <- bind_rows(result, fail_res)
  }

  result <- result[!duplicated(result), ]

  list(tss_df = result, fail_genefilter = fail)
}

library(shiny)
library(readr)

source(system.file("extdata", "primirTSSapp", "extra_fun", "shiny_merge.R", package = "primirTSS"))
options(shiny.maxRequestSize=60*1024^3)

function(input, output) {

  wm_eSet <- eventReactive(input$wm_start, {

	withProgress(
	  message = 'Calculation in progress', detail = 'This may take a while...', value = 0, {

	  incProgress(0.3)
	    if (input$file1_test == "no") {
	      histone_bed <- "NO"
	    } else {
	      histone_bed <- read_csv(input$file1$datapath, guess_max = 100000000)
	    }

	    if (input$file2_test == "no") {
	      pol2_bed <- "NO"
	    } else {
	      pol2_bed <- readr::read_csv(input$file2$datapath, guess_max = 100000000)
	    }

	    if (histone_bed == "NO" && pol2_bed == "NO") {
	      stop("you should input as least one of histone peaks file or pol2 peaks file")
	    } else if (histone_bed == "NO" && pol2_bed != "NO") {
	      bed_merged <- env_shiny$peak_merge2(pol2_bed, n = input$n2)
	    } else if (histone_bed != "NO" && pol2_bed == "NO") {
	      bed_merged <- env_shiny$peak_merge2(histone_bed, n = input$n1)
	    } else if (histone_bed != "NO" && pol2_bed != "NO") {
	      bed_merged_h3 <- env_shiny$peak_merge2(histone_bed, n = input$n1)
	      bed_merged_pol2 <- env_shiny$peak_merge2(pol2_bed, n = input$n2)
	      bed_merged <- peak_join(bed_merged_h3, bed_merged_pol2)
	    }

	    if (input$file3_test == "no") {
	      expressed_mir <- "all"
	    } else {
	      expressed_mir <- readLines(input$file3$datapath)
	    }

	    if (input$file4_test == "yes") {
	      ignore_DHS_check <- FALSE
	      DHS <- readr::read_csv(input$file4$datapath, guess_max = 100000000)
	    } else {
	      ignore_DHS_check <- TRUE
	    }

	    if (input$DHS_check_result == "no") {
	      allmirdhs_byforce <- FALSE
	    } else {
	      allmirdhs_byforce <- TRUE
	    }

	    if (input$file5_test == "no") {
	      expressed_gene <- "all"
	    } else {
	      expressed_gene <- readLines(input$file5$datapath)
	    }

	    if (input$gene_check_result == "no") {
	      allmirgene_byforce <- FALSE
	    } else {
	      allmirgene_byforce <- TRUE
	    }

	    if (input$tf == "yes") {
	      seek_tf <- TRUE
	    } else {
	      seek_tf <- FALSE
	    }
	  incProgress(0.6)

	  tss_result <- env_shiny$find_tss2(bed_merged, expressed_mir = expressed_mir,
	                         flanking_num = input$flanking_num,
	                         threshold = input$threshold,
	                         ignore_DHS_check = ignore_DHS_check,
	                         DHS = DHS,
	                         allmirdhs_byforce = allmirdhs_byforce,
	                         expressed_gene = expressed_gene,
	                         allmirgene_byforce = allmirgene_byforce,
	                         seek_tf = seek_tf, tf_n = input$tf_n,
	                         min.score = input$min.score)

	  incProgress(0.8)

	})
    result <- tss_result$tss_df

    result
  })

  output$wm_exprs_mx <- renderTable({
    head(wm_eSet())
  },
  rownames = FALSE,
  digits = 0)

  output$tss_result.csv <- downloadHandler(
    filename = 'tss_result.csv',
    content = function(file) {
      readr::write_csv(wm_eSet(), file)
    }
  )


  wm_eSet_2 <- eventReactive(input$wm_start.2, {

    withProgress(
      message = 'Calculation in progress', detail = 'This may take a while...', value = 0, {

        incProgress(0.3)
        if (input$file1.2_test == "no") {
          histone_bed <- "NO"
        } else {
          histone_bed <- read_csv(input$file1.2$datapath, guess_max = 100000000)
        }

        if (input$file2.2_test == "no") {
          pol2_bed <- "NO"
        } else {
          pol2_bed <- readr::read_csv(input$file2.2$datapath, guess_max = 100000000)
        }

        if (histone_bed == "NO" && pol2_bed == "NO") {
          stop("you should input as least one of histone peaks file or pol2 peaks file")
        } else if (histone_bed == "NO" && pol2_bed != "NO") {
          bed_merged <- env_shiny$peak_merge2(pol2_bed, n = input$n2.2)
        } else if (histone_bed != "NO" && pol2_bed == "NO") {
          bed_merged <- env_shiny$peak_merge2(histone_bed, n = input$n1.2)
        } else if (histone_bed != "NO" && pol2_bed != "NO") {
          bed_merged_h3 <- env_shiny$peak_merge2(histone_bed, n = input$n1.2)
          bed_merged_pol2 <- env_shiny$peak_merge2(pol2_bed, n = input$n2.2)
          bed_merged <- peak_join(bed_merged_h3, bed_merged_pol2)
        }


        if (input$file4.2_test == "yes") {
          ignore_DHS_check <- FALSE
          DHS <- readr::read_csv(input$file4.2$datapath, guess_max = 100000000)
        } else {
          ignore_DHS_check <- TRUE
        }

        if (input$DHS_check_result.2 == "no") {
          allmirdhs_byforce <- FALSE
        } else {
          allmirdhs_byforce <- TRUE
        }

        if (input$file5.2_test == "no") {
          expressed_gene <- "all"
        } else {
          expressed_gene <- readLines(input$file5.2$datapath)
        }

        if (input$gene_check_result.2 == "no") {
          allmirgene_byforce <- FALSE
        } else {
          allmirgene_byforce <- TRUE
        }

        incProgress(0.6)
        track <- env_shiny$plot_primiRNA2(input$file3.2, bed_merged,
                                     flanking_num = input$flanking_num.2,
                                     threshold = input$threshold.2,
                                     ignore_DHS_check,
                                     DHS, allmirdhs_byforce,
                                     expressed_gene = expressed_gene,
                                     allmirgene_byforce)
      })

    track
  })

  output$wm_plot <- renderPlot({
    wm_eSet_2()

  })

}

#' @importFrom stringr str_detect
#' @importFrom stringr str_split

extand_df <- function(df, colnm, sep = ":") {
  info <- df[[colnm]]
  index <- str_detect(info, sep)
  if (any(index)) {
    double_tss_index <- which(index)
    split_res <- str_split(info[double_tss_index], sep)
    rep_times <- lapply(split_res, length) %>% unlist()
    new_add_seq <- rep(double_tss_index, rep_times)
    new_add_candidate_tss <- unlist(split_res)

    new_seq <- c(seq(length(info))[-double_tss_index], new_add_seq)
    new_info <- c(info[-double_tss_index], new_add_candidate_tss)
  } else {
    new_seq <- seq(length(info))
    new_info <- info
  }
  new_df <- bind_cols(df[new_seq, ], new_info = new_info)
  cmd <- sprintf("new_df <- select(new_df, -%s)", colnm)
  eval(parse(text = cmd))
  new_df
}

#' @importFrom stringr str_split
#' @importFrom tidyr unite
#' @importFrom tidyr separate
#' @importFrom BSgenome.Hsapiens.UCSC.hg38 Hsapiens
#' @importFrom Biostrings reverseComplement
#' @importFrom purrr pmap
#' @importFrom stats runif

cor_tdga <- function(chrom, start, end, strand) {
  a <- Hsapiens[[chrom]]
  if (strand == "+") {
    a[start:end] %>%
      as.character()
  } else {
    a[start:end] %>%
      reverseComplement() %>%
      as.character()
  }
}


require_fa <- function(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                       strand, peak_p1, peak_p2, histone_p1_flank,
                       histone_p2_flank) {

  mir_flank <- tibble::tibble(mir_name = mir_name, chrom = chrom,
                          stem_loop_p1 = stem_loop_p1,
                          stem_loop_p2 = stem_loop_p2, strand = strand,
                          peak_p1 = peak_p1, peak_p2 = peak_p2,
                          histone_p1_flank = histone_p1_flank,
                          histone_p2_flank = histone_p2_flank)
  tcga <- mir_flank %>%
    dplyr::select(chrom, start = histone_p1_flank,
                  end = histone_p2_flank, strand) %>%
    purrr::pmap(cor_tdga) %>%
    unlist()

  line1 <- mir_flank %>%
    mutate(arrow = ">") %>%
    unite(line1,
          mir_name, chrom, stem_loop_p1, stem_loop_p2, strand,
          peak_p1, peak_p2,
          histone_p1_flank, histone_p2_flank, sep = "_") %>%
    unite(line1, arrow, line1, sep = "")

  paste(line1$line1, tcga, sep = "\n")
}


eponine_score <- function(mir_name, chrom, stem_loop_p1, stem_loop_p2,
                          strand, peak_p1, peak_p2, flanking_num = 1000,
                          threshold = 0.7) {
  `%>%` <- magrittr::`%>%`
  mir_peaks <- tibble::tibble(mir_name = mir_name, chrom = chrom,
                          stem_loop_p1 = stem_loop_p1,
                          stem_loop_p2 = stem_loop_p2, strand = strand,
                          peak_p1 = peak_p1, peak_p2 = peak_p2)

  mir_flank <- mir_peaks %>%
    dplyr::mutate(
      histone_p1_flank = peak_p1 - flanking_num,
      histone_p2_flank = peak_p2 + flanking_num
    )

  a <- require_fa(mir_flank$mir_name, mir_flank$chrom,
                  mir_flank$stem_loop_p1, mir_flank$stem_loop_p2,
                  mir_flank$strand, mir_flank$peak_p1, mir_flank$peak_p2,
                  mir_flank$histone_p1_flank, mir_flank$histone_p2_flank)
  tmp_path <- file.path(tempdir(), paste0(runif(1, 0, 100), ".fa"))
  writeLines(a, tmp_path)

  java_path <- system.file("extdata", "eponine-scan.jar", package = "primirTSS")
  cmd <- sprintf("java -jar %s -seq %s -threshold %s",
                 java_path, tmp_path, threshold)
  aa <- system(cmd, intern = TRUE)

  file.remove(tmp_path)

  a_tmp <- aa[-seq_len(3)] %>%
    str_split("\t", simplify = TRUE)

  result <- tibble::tibble(previous = a_tmp[, 1],
                       tss_p1 = a_tmp[, 4],
                       tss_p2 = a_tmp[, 5],
                       eponine_score = a_tmp[, 6]) %>%
    separate(previous,
             into = c("mir_name", "chrom", "stem_loop_p1", "stem_loop_p2",
                      "strand",
                      "peak_p1", "peak_p2",
                      "histone_p1_flank", "histone_p2_flank"),
             sep = "_") %>%
    mutate(stem_loop_p1 = as.double(stem_loop_p1),
           stem_loop_p2 = as.double(stem_loop_p2),
           peak_p1 = as.double(peak_p1),
           peak_p2 = as.double(peak_p2),
           histone_p1_flank = as.double(histone_p1_flank),
           histone_p2_flank = as.double(histone_p2_flank),
           tss_p1 = as.double(tss_p1),
           tss_p2 = as.double(tss_p2),
           eponine_score = as.double(eponine_score)) %>%
    mutate(tss_p1 = histone_p1_flank + tss_p1 - 1,
           tss_p2 = histone_p1_flank + tss_p2 - 1) %>%
    filter(tss_p1 >= peak_p1 & tss_p2 <= peak_p2) %>%
    select(-(peak_p1:histone_p2_flank))

  fail_mir <- setdiff(unique(mir_peaks$mir_name), unique(result$mir_name))

  if (length(fail_mir) == 0) {
    fail <- "All miRNAs have eponine socres"
  } else {
    fail <- fail_mir
  }

  list(success = result, fail_eponine = fail)
}

---
title: "primirTSS"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{primirTSS}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## 1 Introduction
Identifying human miRNA transcriptional start sites (TSSs) plays a significant 
role in understanding the transcriptional regulation of miRNA. However, due to 
the quick capping of pri-miRNA and many miRNA genes may lie in the introns or 
even exons of other genes, it is difficult to detect miRNA TSSs. miRNA TSSs are 
cell-specific. And miRNA TSSs are cell-specific, which implies the same miRNA in 
different cell-lines may start transcribing at different TSSs.

High throughput sequencing, like ChIP-seq, has gradually become an essential and 
versatile approach for us to identify and understand genomes and their 
transcriptional processes. 
By integrating H3k4me4 and Pol II data, parting of false positive counts 
after scoring can be filtered out. Besides, DNase I hypersensitive sites(DHS) also
imply TSSs, where miRNAs will be accessible and functionally related to 
transcription activities. And additionally, the expression profile of miRNA and 
genes in certain cell-line will be considered as well to improve fidelity. By 
employing all these different kinds of data, here we have developed the primirTSS 
package to assist users to identify miRNA TSSs in human and to provide them with 
related information about where miRNA genes lie in the genome, with both 
command-line and graphical interfaces.


## 2 Find the best putative TSS

### Installation

#### 1 primirTSS

Install the latest release of R, then get `primirTSS` by starting R and entering 
the commands:

```{r, eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")
BiocManager::install("primirTSS")
```

Or install the development version of the package using the same command, 
but with GitHub account name.

```{r, eval = FALSE}
BiocManager::install("ipumin/primirTSS")
```

#### 2 Install Java SE Development Kit(JDK)
As Java development environment is indispensable for the primary function in our 
package, it is <span style="border-bottom:1px dashed black;">necessary</span> 
for users to install [Java SE Development Kit 10](http://www.oracle.com/technetwork/java/javase/downloads/jdk10-downloads-4416644.html) 
before using `primirTSS`. 

#### 3 Load the package into R session

```{r Load, message=FALSE}
library(primirTSS)
```


## 3 Getting Started

### Step 1: Process of H3K4me3 and Pol II data

* `peak_merge()`: **Merge one kind of peaks** (H3K4me3 **or** Pol II)

**H3K4me3** and **Pol II** data are key points for accurate prediction with our method. 
If one of these two peak data is input, before executing the main function `find_TSS`, 
the function `peak_merge` should be used to merge adjacent peaks whose distance 
between each other is less than `n` base pairs and return the merged peaks as an 
output.

```{r}
library(primirTSS)
peak_df <- data.frame(chrom = c("chr1", "chr2", "chr1"),
                       chromStart = c(450, 460, 680),
                       chromEnd = c(470, 480, 710),
                       stringsAsFactors = FALSE)
peak <-  as(peak_df, "GRanges")
peak_merge(peak, n =250)
```

</br>

* `peak_join()`: **Join two kinds of peaks** (H3K4me3 **and** Pol II)
 
If both of H3K4me3 and Pol II data, after separately merging these two kinds of
evidence first, `peak_join` should be employed to integrate H3K4me3 and Pol II peaks
and return the result as `bed_merged` parameter for the main function `find_tss`.

```{r}
peak_df1 <- data.frame(chrom = c("chr1", "chr1", "chr1", "chr2"),
                       start = c(100, 460, 600, 70),
                       end = c(200, 500, 630, 100),
                       stringsAsFactors = FALSE)
peak1 <-  as(peak_df1, "GRanges")

peak_df2 <- data.frame(chrom = c("chr1", "chr1", "chr1", "chr2"),
                       start = c(160, 470, 640, 71),
                       end = c(210, 480, 700, 90),
                       stringsAsFactors = FALSE)
peak2 <-  as(peak_df2, "GRanges")

peak_join(peak1, peak2)
```

### Step 2: Predict most possible TSS for miRNA


* `find_tss` is the primary function in the package. The program will first score 
the candidate TSSs of miRNA and pick up the best candidate in the first step of 
prediction, (where users can set `flanking_num` and `threshold`). 
* After the first step, H3K4me3 and Pol II data, miRNA expression profiles and
DHS check, protein-coding genes expression profiles, if provided, will be 
integrated to decrease the rate of false positive.

</br>
</br>

There will be different circumstances where not all miRNA expression profiles, 
DHS data, protein-coding gene('gene') expression profiles are available:

</br>

**Circumstance 1:** no miRNA expression data; then suggest DHS check and 
protein-coding gene check.

* `ignore_DHS_check`: If users do not have their own miRNA expression profile, 
the function will employ all the miRNAs already annotated in human, but we 
suggest using DHS data of the cell line from **ENCODE** to check whether this
miRNA is expressed in the cell line or not as well as and all human gene expression 
profiles from **Ensemble** to check the relative position of TSSs and 
protein-coding genes to improve the accuracy of prediction.


```{r}
peakfile <- system.file("testdata", "HMEC_h3.csv", package = "primirTSS")
DHSfile <- system.file("testdata", "HMEC_DHS.csv", package = "primirTSS")
peak_h3 <- read.csv(peakfile, stringsAsFactors = FALSE)
DHS <- read.csv(DHSfile, stringsAsFactors = FALSE)
DHS <- as(DHS, "GRanges")
peak_h3 <-  as(peak_h3, "GRanges")
peak <- peak_merge(peak_h3)
```
```
no_ownmiRNA <- find_tss(peak, ignore_DHS_check = FALSE,
                        DHS = DHS, allmirdhs_byforce = FALSE,
                        expressed_gene = "all",
                        allmirgene_byforce = FALSE,
                        seek_tf = FALSE)
```

</br>

**Circumstance 2**: miRNA expression data provided; then no need for DHS check 
but protein-coding gene check.

* `expressed_mir`: If users have their own miRNA expression profiles, we will use 
the expressed miRNAs and we suggest not using DHS data of the cell line or others 
to check the expression of miRNAs.But the protein-coding gene check to check the 
relative position of TSSs and protein-coding genes is necessary, which helps to
verify the precision of prediction.

```{r}
bed_merged <- data.frame(
                chrom = c("chr1", "chr1", "chr1", "chr1", "chr2"),
                start = c(9910686, 9942202, 9996940, 10032962, 9830615),
                end = c(9911113, 9944469, 9998065, 10035458, 9917994),
                stringsAsFactors = FALSE)
bed_merged <- as(bed_merged, "GRanges")

expressed_mir <- c("hsa-mir-5697")

ownmiRNA <- find_tss(bed_merged, expressed_mir = expressed_mir,
                     ignore_DHS_check = TRUE,
                     expressed_gene = "all",
                     allmirgene_byforce = TRUE,
                     seek_tf = FALSE)
```

</br>

* `expressed_gene`: Additionally, users can also specify certain genes expressed
in the cell-line being analyzed:



### Step 3: Searching for TFs
* `seek_tf = TRUE`: If user want to predict transcriptional regulation 
relationship between TF and miRNA, like which TFs might regulate miRNA after get 
TSSs, they can change `seek_tf = FALSE` from `seek_tf = TRUE` directly in the 
comprehensive function `find_TSS()`. 


### Step4: Analysis of results 


Here is a demo of predicting TSS for hsa-mir-5697, ignore DHS check.



</br>

**PART1**, `$tss_df`:

```{r}
ownmiRNA$tss_df
```


The first part of the result returns details of predicted TSSs, composed of 
seven columns: 
*mir\_name, chrom, stem\_loop\_p1, stem\_loop\_p2, strand mir\_context, tss\_type gene* 
and *predicted_tss*:

 Entry|Implication
 :----:|----
 **mir_name**| Name of miRNA.
 **chrom** | Chromosome.
 **stem\_loop_p1**| The start site of a stem-loop.
 **stem\_loop_p2**| The end site of a stem-loop.
 **strand**|Polynucleotide strands. (`+/-`)
 **mir_context**|2 types of relative position relationship between stem-loop and protein-coding gene. (`intra/inter`)
 **tss_type**|4 types of predicted TSSs. See the section below TSS types for details.(`host_TSS/intra_TSS/overlap_inter_TSS/inter_TSS`)
 **gene**|Ensembl gene ID.
 **predicted_tss**| Predicted transcription start sites(TSSs).
 **pri\_tss_distance**|The distance between a predicted TSS and the start site of the stem-loop.
 
 
TSSs are cataloged into 4 types as below:

 * **host_TSS**: The TSSs of miRNA that are close to the TSS of protein-coding gene
  implying they may share the same TSS,
  on the condition where `mir_context = intra`.
  (See above: `mir_context`)

 * **intra_TSS:** The TSSs of miRNA that are NOT close to the TSS of protein-coding gene,
  on the condition where `mir_context = intra`.

 * **overlap\_inter_TSS:** The TSSs of miRNA are cataloged as `overlap_inter_TSS` 
 when the pri-miRNA gene overlaps with Ensembl gene, on the condition where "`mir_context = inter`".
  
 * **inter\_inter_TSS:** The TSSs of miRNA are cataloged as `inter_inter_TSS`
  when the miRNA gene does NOT overlap with Ensembl gene,
  on the condition where "`mir_context = inter`".

  (See [Xu HUA et al](https://academic.oup.com/bioinformatics/article-lookup/doi/10.1093/bioinformatics/btw171) 2016 for more details)


</br>


**PART2**, `$log`:

The second part of the result returns **4 logs** created during the process of prediction:

* **`find_nearest_peak_log`**:
  If no peaks locate in the upstream of
  a stem-loop to help determine putative TSSs of miRNA,
  we will fail to find the nearest peak,
  and this miRNA will be logged in `find_nearest_peak_log`.

 * **`eponine_score_log`**:
  For a certain miRNA, if none of the candidate TSSs scored with
Eponine method meet the threshold we set,
  we will fail to get an eponine score,
  and this miRNA will be logged in `eponine_score_log`.

 * **`DHS_check_log`**:
  For a certain miRNA, if no DHS signals locate
  within 1 kb upstream of each putative TSSs,
  these putative TSSs will be filtered out,
  and this miRNA will be logged in `DHS_check_log`.

 * **`gene_filter_log`**:
  For a certain miRNA, when integrating expressed_gene data to improve prediction,
  if no putative TSSs are confirmed after considering the relative
  position relationship among TSSs, stem-loops and expressed genes,
  this miRNA will be filtered out and logged in `gene_filter_log`.


</br>
</br>

## 4 Plot the prediction of TSS for miRNA

* `plot_primiRNA()`: Apart from returning the putative TSS of each miRNA, the 
package `primirTSS` can also visualize the result and return an image composed 
of six tracks, (1)TSS, (2)genome, (3)pri-miRNA, (4)the closest gene, (5)eponine score and
(6)conservation score.
And the parameters in this function is almost the same as those in `find_tss()` 
except `expressed_mir` only represents one certain miRNA in `plot_primiRNA()`. 
**NOTICE** that this function is used for visualizing the TSS prediction of 
**only one** specific miRNA every single time. 

```
plot_primiRNA(expressed_mir, bed_merged,
              flanking_num = 1000, threshold = 0.7,
              ignore_DHS_check = TRUE,
              DHS, allmirdhs_byforce = TRUE,
              expressed_gene = "all",
              allmirgene_byforce = TRUE)
```

</br>

![plot_tss](../inst/extdata/pic/plot_tss.jpg)
　
　Figure S1. _**Visualized result for miRNA TSSs by `Plot pri-miRNA TSS()`**_

>As Figure S1 shows, the picture contains information of the pri-miRNA’s 
coordinate, the closest gene to the miRNA, the eponine score of the miRNA’s 
candidate TSS and the conservation score of the miRNA’s candidate TSS.
>There are six tracks plotted in return:
>
>
> Entry|Implication
> :----:|-----
> **Chromosome** |Position of miRNA on the chromosome.
> **hg19** |Reference genome coordinate in hg19.
> **pri-miRNA**: |Position of pri-miRNA.
> **Ensemble genes** |Position of related protein-coding gene.
> **Eponine score** |Score of best putative TSS by Eponine method.
> **Conservation score** |Conservation score of TSS. 　　　　　　　　　　　　　　　　　　　　　　　　

</br>
</br>

## 5 Graphical web interface for prediction


* `run_primirTSSapp()`: A graphical web interface is designed to achieve the 
functions of `find_tss` and `plot_primiRNA` to help users intuitively and 
conveniently predict putative TSSs of miRNA. Users can refer documents of the 
two functions, 
**Find the best putative TSS** and **Plot the prediction of TSSs for miRNA**, 
mentioned above for details.
</br>


### TAG1: Find the best putative TSS

![shiny_tss](../inst/extdata/pic/shiny_tss.jpg)
　Figure S2. _**Graphical web interface of `Find pri-miRNA TSS()`**_


>As Figure S2 shows, if we want to use the shiny app, we should select the 
appropriate options or upload the appropriate files. Histone peaks, Pol II 
peaks and DHS files are comma-separated values (CSV) files, whose first line 
is chrom,start,end. Every line of miRNA expression profiles has only one miRNA 
name which start with hsa-mir, such as hsa-mir-5697. Every line of gene expression 
profiles has only one gene name which derived from Ensembl, such as ENSG00000261657. 
All of miRNA expression profiles and gene expression profiles do not have column 
names. If we have prepared, we can push the Start the analysis button to start 
finding the TSSs. The process of analysis may need to take a few minutes, and a
process bar will appear in right corner.
>
>As a result, we will view first six rows of the result. The first five columns 
are about <span style="border-bottom:1px dashed black;"> miRNA </span> information,
next five columns are about <span style="border-bottom:1px dashed black;"> 
TSS </span> information. The column of gene denotes the gene whose TSS is 
closest to the miRNA TSS. The column of pri\_tss\_distance denotes the distance 
between miRNA TSS and stem-loop. If users choose to get TFs simultaneously, 
they will have an additional column, `tf`, which stores related TFs.

</br>

### TAG2: Plot pri-miRNA TSS

![shiny_plot](../inst/extdata/pic/shiny_plot.jpg)
　Figure S3. _**Graphical web interface of `Plot pri-miRNA TSS()`**_


>As Figure S4 shows, if we select the appropriate options and upload the 
appropriate files, we can have a picture of miRNA TSSs.

</br>

## Session info
Here is the output of sessionInfo() on the system on which this document was compiled:

```{r}
sessionInfo()
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/transform_cor.R
\name{trans_cor}
\alias{trans_cor}
\title{transform one hg coordinates to another}
\usage{
trans_cor(peak, hg_from, hg_to)
}
\arguments{
\item{peak}{A \code{GRange} object. The genome, the coordinates of which need
to be coverted.}

\item{hg_from}{The genome are coverting from. This parameter can be "hg18",
"hg19" or "hg38", etc.}

\item{hg_to}{Which type the genome is converting to. This parameter can be "hg18",
"hg19" or "hg38", etc. NOTICE \code{hg_from} and \code{hg_to} should be
different from each other.}
}
\value{
A GRanges object.
}
\description{
Convert coordinates between different genomes when necessary.
}
\examples{

peak_df <- data.frame(chrom = c("chr7", "chr7", "chr7"),
                      chromStart = c(128043908, 128045075, 128046242),
                      chromEnd = c(128045074, 128046241, 128047408),
                      stringsAsFactors = FALSE)
peak <-  as(peak_df, "GRanges")

trans_cor(peak, "hg19", "hg38")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primirTSS-package.R
\docType{package}
\name{primirTSS}
\alias{primirTSS}
\title{primirTSS: Search for putative TSSs of miRNA}
\description{
A fast, convenient tool to identify the TSSs of miRNAs
by integrating the data of H3K4me3 and Pol II as well as
combining the conservation level and sequence feature,
provided within both command-line and graphical interfaces,
which achieves a better performance than
the previous non-cell-specific methods on miRNA TSSs.
}
\section{Detail}{

  See \code{\link{find_tss}} for deailted instruction
  to predict TSSs of miRNA;
  See \code{\link{run_primirTSSapp}}
  to predict TSSs of miRNA using graphical web interface.
}

\section{Author(s)}{


  Maintainer: Pumin Li \email{ipumin@163.com}

  Other contributors: Qi Xu \email{xuqi@vip.qq.com}
}

\seealso{
Useful links:
  \href{https://academic.oup.com/bioinformatics/article-lookup/doi/10.1093/bioinformatics/btw171}{Xu HUA et al}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_miRNA_track.R
\name{plot_primiRNA}
\alias{plot_primiRNA}
\title{Plot the result of prediction for miRNA}
\usage{
plot_primiRNA(
  expressed_mir,
  bed_merged,
  flanking_num = 1000,
  threshold = 0.7,
  ignore_DHS_check = TRUE,
  DHS,
  allmirdhs_byforce = TRUE,
  expressed_gene = "all",
  allmirgene_byforce = TRUE
)
}
\arguments{
\item{expressed_mir}{This parameter allows users to specify certain miRNAs,
the TSSs of which they want to search for by providing a list of
miRNAs(e.g. expressed miRNAs in a certain cell-line). If
\code{expressed_mir} is not specified, the default value of the parameter
is "\code{all}" and the function will acquiescently employ all the miRNAs
currently listed on "\code{miRbase}" database.}

\item{bed_merged}{Peaks from ChIP-seq data to be provided for analysis can be
H3K4me3 peaks, Pol II peaks or both. Notice that peaks are supposed to be
merged(see also \code{\link{peak_merge}}) before \code{find_TSS} if using
only one kind of peak data, while peaks shoud be firstly merged and then
join together(see also \code{\link{peak_join}}) if both H3K4me3 data and
Pol II are input.}

\item{flanking_num}{A parameter in Eponine model to detect TSSs. It is
concluded that a peak signal with flanking regions of C-G enrichment are
important to mark TSSs. The default value is 1000.}

\item{threshold}{Threshold for candidate TSSs scored with Eponine method. The
default value is 0.7.}

\item{ignore_DHS_check}{The process of DHS_check further assist to filter
putative TSSs. When there are a DHS peak that locates within 1 kb upstream
of a putative TSS, this predicted TSS will be retain for it character is
consistent with that of an authentic TSS. Or the TSSs with no DHSs locating
within 1 kb upstream of them would be discard.}

\item{DHS}{ChIP-seq data of DNase I hypersensitive sites(DHSs).}

\item{allmirdhs_byforce}{When we use DHS data to check the validity of TSSs,
there is possibility where no DHSs locates within 1 kb upstream of all
putative TSSs and all these putative TSSs might be filtered out by our
method resulting no outputs. While "\code{allmirdhs_byforce} = TRUE", it
ensures to output at least 1 most possible TSS even if the nearest DHS
signal locates more than 1 kb upsteam of this TSS.}

\item{expressed_gene}{Users can speicify genes expressed in certain
cell-lines that is analyzed. Or the default value is "\code{all}", which
means all the expressed genes annotated on Ensemble will be employed.}

\item{allmirgene_byforce}{While integrating expressed_gene data to improve
prediction, there might be a circumstance where all the putative TSS are
discarded. To prevent this condition, users are allowed to use
"\code{allmirgene_byforce = TRUE}" to ensure at least 1 putative TSS for
each miRNA will be output.}
}
\value{
There will be six tracks plotted as return:

\code{Chrom}: Position of miRNA on the chromosome.

\code{hg38}: Reference genome coordinate in hg38.

\code{pri-miRNA}: Position of pri-miRNA.

\code{Ensemble genes}: Position of related protein-coding gene.

\code{eponine score}: Score of best putative TSS conducted by eponine method.

\code{conservation score}: Conservation score should be integrated with
eponine score to find out putative TSSs.
}
\description{
For each miRNA, plot the position of TSS, pri-miRNA, related Ensemble gene,
eponine socre and conservation score according to the result of prediction
using primirTSS.
}
\details{
NOTICE that this function is used for visualizing the predicted result of
ONLY ONE specific miRNA every single time.
}
\examples{
expressed_mir <- "hsa-mir-5697"
bed_merged <- data.frame(
  chrom = c("chr1", "chr1", "chr1", "chr1", "chr2"),
  start = c(9180799, 9201483, 9234339, 9942202, 9830615),
  end = c(9183889, 9202580, 9235853, 9944469, 9917994),
  stringsAsFactors = FALSE
)
bed_merged <- as(bed_merged, "GRanges")
\dontrun{
plot_primiRNA(expressed_mir, bed_merged)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/primirTSS_shiny.R
\name{run_primirTSSapp}
\alias{run_primirTSSapp}
\title{Predict TSSs of miRNA using a graphical web interface.}
\usage{
run_primirTSSapp()
}
\value{
A graphical interface.
}
\description{
A graphical web interface is provided for users to achieve the functions of
\code{\link{find_tss}} and \code{\link{plot_primiRNA}} to intuitively and
conveniently predict putative TSSs of miRNA.
}
\details{
Users can refer documents of the two functions mentioned ABOVE for details.
}
\examples{

\dontrun{
run_primirTSSapp()
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peak_join.R
\name{peak_join}
\alias{peak_join}
\title{Integrate H3K4me3 data and Pol II data.}
\usage{
peak_join(peak1, peak2)
}
\arguments{
\item{peak1}{H3K4me3 peaks. Merged peak data as \code{GRange} object by
fucntion \code{peak_merge}}

\item{peak2}{Pol II peaks. Merged peak data as \code{GRange} object by
fucntion \code{peak_merge}}
}
\value{
A GRanges object. The joined peaks for the following analysis to
  search for TSSs.
}
\description{
Integrate peaks from H3K4me3 and Pol II data. To conduct the overlappd ranges
for the further analysis by imposing H3K4me3 peaks on Pol II peaks, if both
of these two different kinds of ChIP-seq data are available.
}
\section{Detail}{
 Peak1 and peak2 are signals seperately from the ChIP-seq
  data of H3K4me3 and Pol II data that to be integrated. The data is
  \code{GRange} object containing three columns \code{Chrom}, \code{Ranges},
  \code{Strand}. And the order of these two kinds of data when input as
  \code{peak1} and \code{peak2} can be swapped.
}

\examples{
peak_df1 <- data.frame(chrom = c("chr1", "chr1", "chr1", "chr2"),
                       start = c(100, 460, 600, 70),
                       end = c(200, 500, 630, 100),
                       stringsAsFactors = FALSE)
peak1 <-  as(peak_df1, "GRanges")

peak_df2 <- data.frame(chrom = c("chr1", "chr1", "chr1", "chr2"),
                       start = c(160, 470, 640, 71),
                       end = c(210, 480, 700, 90),
                       stringsAsFactors = FALSE)
peak2 <-  as(peak_df2, "GRanges")

peak_join(peak1, peak2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/peak_merge.R
\name{peak_merge}
\alias{peak_merge}
\title{Merge adjacent peaks within H3K4me3 or Pol II data.}
\usage{
peak_merge(peak, n = 250)
}
\arguments{
\item{peak}{A \code{GRange} object. The peaks to be merged from one certain
ChIP-seq data, such as H3K4me3 data or Pol II data.}

\item{n}{A number. \code{n} stipulates the distance(bp, base pair) between
two seperate peaks within which they should be merged.}
}
\value{
A GRanges object. The merged peaks for the following analysis to
  search for TSSs.
}
\description{
Merge the adjacent segments provided as \code{GRange} object from original
data. This function will merge adjacent peaks the distance between which is
less than n base pairs apart and then return the merged segments.
}
\examples{
peak_df <- data.frame(chrom = c("chr1", "chr2", "chr1"),
                      chromStart = c(450, 460, 680),
                      chromEnd = c(470, 480, 710),
                      stringsAsFactors = FALSE)
peak <-  as(peak_df, "GRanges")

peak_merge(peak, n =250)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_tss.R
\name{find_tss}
\alias{find_tss}
\title{Predict TSSs of miRNA}
\usage{
find_tss(
  bed_merged,
  expressed_mir = "all",
  flanking_num = 1000,
  threshold = 0.7,
  ignore_DHS_check = TRUE,
  DHS,
  allmirdhs_byforce = TRUE,
  expressed_gene = "all",
  allmirgene_byforce = TRUE,
  seek_tf = FALSE,
  tf_n = 1000,
  min.score = 0.8
)
}
\arguments{
\item{bed_merged}{Peaks from ChIP-seq data to be provided for analysis can be
H3K4me3 peaks, Pol II peaks or both. Notice that peaks are supposed to be
merged(see also \code{\link{peak_merge}}) before \code{find_TSS} if using
only one kind of peak data, while peaks should be firstly merged and then
join together(see also \code{\link{peak_join}}) if both H3K4me3 data and
Pol II are input.}

\item{expressed_mir}{This parameter allows users to specify certain miRNAs,
the TSSs of which they want to search for by providing a list of
miRNAs(e.g., expressed miRNAs in a certain cell-line). If
\code{expressed_mir} is not specified, the default value of the parameter
is "\code{all}" and the function will acquiescently employ all the miRNAs
currently listed on "\code{miRbase}" database.}

\item{flanking_num}{A parameter in Eponine model to detect TSSs. It is
concluded that a peak signal with flanking regions of C-G enrichment are
important to mark TSSs. The default value is 1000.}

\item{threshold}{The threshold for candidate TSSs scored with Eponine method. The
default value is 0.7.}

\item{ignore_DHS_check}{The process of DHS_check further assists to filter
putative TSSs. When there is a DHS peak that locates within 1 kb upstream
of a putative TSS, this predicted TSS will be retained for its character is
consistent with that of an authentic TSS. Or the TSSs with no DHSs locating
within 1 kb upstream of them would be discarded.}

\item{DHS}{ChIP-seq data of DNase I hypersensitive sites(DHSs).}

\item{allmirdhs_byforce}{When we use DHS data to check the validity of TSSs,
there is a possibility where no DHSs locates within 1 kb upstream of all
putative TSSs and all these putative TSSs might be filtered out by our
method resulting no outputs. While "\code{allmirdhs_byforce} = TRUE", it
ensures to output at least 1 most possible TSS even if the nearest DHS
signal locates more than 1 kb upstream of this TSS.}

\item{expressed_gene}{Users can specify genes expressed in certain
cell-lines that are analyzed. Or the default value is "\code{all}", which
means all the expressed genes annotated on Ensemble will be employed.}

\item{allmirgene_byforce}{While integrating expressed_gene data to improve
prediction, there might be a circumstance where all the putative TSS are
discarded. To prevent this condition, users are allowed to use
"\code{allmirgene_byforce = TRUE}" to ensure at least 1 putative TSS for
each miRNA will be output.}

\item{seek_tf}{With the result of predicted TSSs, seek_tf provides users with
an option to predict related TFs for miRNA. The data of transcription
factors refer to \code{JASPAR2018} database.}

\item{tf_n}{TFBS locates on the upstream of the TSS of a certain TF, which is
considered as the promoter region. \code{tf_n} set the length of promoter
region for predicting transcription regulation between miRNAs and TFs.}

\item{min.score}{The threshold for scoring transcription factor binding sites. A
single absolute value between 0 and 1.}
}
\value{
The first part of the result returns details of predicted TSSs,
  composed of seven columns: \code{mir_name, chrom, stem_loop_p1,
  stem_loop_p2, strand mir_context, tss_type gene and predicted_tss}:

  \code{mir_name}: Name of miRNA.

  \code{chrom}: Chromosome.

  \code{stem_loop_p1}: The start site of a stem-loop.

  \code{stem_loop_p2}: The end site of a stem-loop.

  \code{strand}: Polynucleotide strands. (\code{+/-})

  \code{mir_context}: The relative positon relationship between stem-loop and
  protein-coding gene. (\code{intra/inter})

  \code{tss_type}: Four types of predicted TSSs. See the section below
  \code{TSS types} for details.
  (\code{host_TSS/intra_TSS/overlap_inter_TSS/inter_TSS})

  \code{gene}: Ensembl gene ID

  \code{predicted_tss}: Predicted transcription start sites(TSSs).

  \code{pri_tss_distance}: The distance between a predicted TSS and the start
  site of the stem-loop.
}
\description{
Search for putative TSSs of miRNA, together with integrating available data such
as H3K4me3 data, Pol II data, miRNA expression data, and protein-coding gene
data, as well as provide the transcriptional regulation relationship between TF
and miRNA.
}
\section{TSS types}{
 TSSs are catalogued into 4 types as below.

  \code{host_TSS} The TSSs of miRNA that are close to the TSS of
  protein-coding gene implying they may share the same TSS, on the condition
  where \code{mir_context} is "\code{intra}". (See above:
  \code{Value-mir_context})

  \code{intra_TSS} The TSSs of miRNA that are NOT close to the TSS of
  the protein-coding gene, on the condition where \code{mir_context} is
  "\code{intra}".

  \code{overlap_inter_TSS} The TSSs of miRNA are catalogued as
  "\code{overlap_inter_TSS}" when the pri-miRNA gene overlaps with Ensembl
  gene, on the condition where "\code{mir_context}" is "\code{inter}".

  \code{inter_inter_TSS} The TSSs of miRNA are catalogued as
  "\code{inter_inter_TSS}" when the miRNA gene does NOT overlap with Ensembl
  gene, on the condition where "\code{mir_context}" is "\code{inter}".

  (See Xu HUA et al 2016 for more details)
}

\section{Log}{


  The second part of the result returns logs during the process of
  prediction: \code{find_nearest_peak_log} If no peaks locate in the
  upstream of a stem-loop to help determine putative TSSs of miRNA, we will
  fail to find the nearest peak and this miRNA will be logged in
  \code{find_nearest_peak_log}.

  \code{eponine_score_log} For a certain miRNA, if none of the candidate
  TSSs scored with \code{Eponine method} meet the threshold we set, we will
  fail to get an eponine score and this miRNA will be logged in
  \code{eponine_score_log}.

  \code{DHS_check_log} For a certain miRNA, if no DHS signals locate
  within 1 kb upstream of each putative TSSs, these putative TSSs will be
  filtered out and this miRNA will be logged in \code{DHS_check_log}.

  \code{gene_filter_log} For a certain miRNA, when integrating
  expressed_gene data to improve prediction, if no putative TSSs are
  confirmed after considering the relative position relationship among TSSs,
  stem-loops and expressed genes, this miRNA will be filtered out and logged
  in \code{gene_filter_log}.
}

\section{Reference}{
 Xu Hua, Luxiao Chen, Jin Wang*, Jie Li* and Edgar
  Wingender*, Identifying cell-specific microRNA transcriptional start sites.
  Bioinformatics 2016, 32(16), 2403-10.
}

\examples{

bed_merged <- data.frame(
                chrom = c("chr1", "chr1", "chr1", "chr1", "chr2"),
                start = c(9910686, 9942202, 9996940, 10032962, 9830615),
                end = c(9911113, 9944469, 9998065, 10035458, 9917994),
                stringsAsFactors = FALSE)
bed_merged <- as(bed_merged, "GRanges")

\dontrun{
ownmiRNA <- find_tss(bed_merged, expressed_mir = "hsa-mir-5697",
                     ignore_DHS_check = TRUE,
                     expressed_gene = "all",
                     allmirgene_byforce = TRUE)
}

}
